from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

okta_login_without_push_marker_tests: list[RuleTest] = [
    RuleTest(
        name="Login with marker",
        expected_result=False,
        log={
            "actor": {
                "alternateId": "alice.beaver@company.com",
                "displayName": "Alice Beaver",
                "id": "00u99ped55av2JpGs5d7",
                "type": "User",
            },
            "authenticationContext": {"authenticationStep": 0, "externalSessionId": "trsxcsf59kYRG-GwAbWjw-PZA"},
            "client": {
                "device": "Unknown",
                "ipAddress": "11.22.33.44",
                "userAgent": {"browser": "UNKNOWN", "os": "Unknown", "rawUserAgent": "Go-http-client/2.0 PS_mxzqarw"},
                "zone": "null",
            },
            "debugContext": {
                "debugData": {
                    "dtHash": "53dd1a7513e0256eb13b9a47bb07ed61e8ca3d35fbdc36c909567a21a65a2b19",
                    "rateLimitBucketUuid": "b192d91c-b242-36da-9332-d97a5579f865",
                    "rateLimitScopeType": "ORG",
                    "rateLimitSecondsToReset": "6",
                    "requestId": "234cf34e0081e025e1fe14224464bbd6",
                    "requestUri": "/api/v1/logs",
                    "threshold": "20",
                    "timeSpan": "1",
                    "timeUnit": "MINUTES",
                    "url": "/api/v1/logs?since=2023-09-21T17%3A04%3A22Z&limit=1000&after=1714675441520_1",
                    "userId": "00u99ped55av2JpGs5d7",
                    "warningPercent": "60",
                },
            },
            "displayMessage": "Rate limit warning",
            "eventType": "system.org.rate_limit.warning",
            "legacyEventType": "core.framework.ratelimit.warning",
            "outcome": {"result": "SUCCESS"},
            "published": "2024-05-02 18:46:21.121000000",
            "request": {"ipChain": [{"ip": "11.22.33.44", "version": "V4"}]},
            "securityContext": {},
            "severity": "WARN",
            "target": [
                {"id": "/api/v1/logs", "type": "URL Pattern"},
                {"id": "b192d91c-b242-36da-9332-d97a5579f865", "type": "Bucket Uuid"},
            ],
            "transaction": {
                "detail": {"requestApiTokenId": "00T1bjatrp6Nl1dOc5d7"},
                "id": "234cf34e0081e025e1fe14224464bbd6",
                "type": "WEB",
            },
            "uuid": "44aeb388-08b4-11ef-9cec-73ffcb6f9fdd",
            "version": "0",
        },
    ),
    RuleTest(
        name="Login without marker",
        expected_result=True,
        log={
            "actor": {
                "alternateId": "alice.beaver@company.com",
                "displayName": "Alice Beaver",
                "id": "00u99ped55av2JpGs5d7",
                "type": "User",
            },
            "authenticationContext": {"authenticationStep": 0, "externalSessionId": "trsxcsf59kYRG-GwAbWjw-PZA"},
            "client": {
                "device": "Unknown",
                "ipAddress": "11.22.33.44",
                "userAgent": {"browser": "UNKNOWN", "os": "Unknown", "rawUserAgent": "Go-http-client/2.0"},
                "zone": "null",
            },
            "debugContext": {
                "debugData": {
                    "dtHash": "53dd1a7513e0256eb13b9a47bb07ed61e8ca3d35fbdc36c909567a21a65a2b19",
                    "rateLimitBucketUuid": "b192d91c-b242-36da-9332-d97a5579f865",
                    "rateLimitScopeType": "ORG",
                    "rateLimitSecondsToReset": "6",
                    "requestId": "234cf34e0081e025e1fe14224464bbd6",
                    "requestUri": "/api/v1/logs",
                    "threshold": "20",
                    "timeSpan": "1",
                    "timeUnit": "MINUTES",
                    "url": "/api/v1/logs?since=2023-09-21T17%3A04%3A22Z&limit=1000&after=1714675441520_1",
                    "userId": "00u99ped55av2JpGs5d7",
                    "warningPercent": "60",
                },
            },
            "displayMessage": "Rate limit warning",
            "eventType": "system.org.rate_limit.warning",
            "legacyEventType": "core.framework.ratelimit.warning",
            "outcome": {"result": "SUCCESS"},
            "published": "2024-05-02 18:46:21.121000000",
            "request": {"ipChain": [{"ip": "11.22.33.44", "version": "V4"}]},
            "securityContext": {},
            "severity": "WARN",
            "target": [
                {"id": "/api/v1/logs", "type": "URL Pattern"},
                {"id": "b192d91c-b242-36da-9332-d97a5579f865", "type": "Bucket Uuid"},
            ],
            "transaction": {
                "detail": {"requestApiTokenId": "00T1bjatrp6Nl1dOc5d7"},
                "id": "234cf34e0081e025e1fe14224464bbd6",
                "type": "WEB",
            },
            "uuid": "44aeb388-08b4-11ef-9cec-73ffcb6f9fdd",
            "version": "0",
        },
    ),
]


@panther_managed
class OktaLoginWithoutPushMarker(Rule):
    id = "Okta.Login.Without.Push.Marker-prototype"
    display_name = "Okta Login Without Push Marker"
    enabled = False
    tags = ["Push Security", "Configuration Required"]
    log_types = [LogType.OKTA_SYSTEM_LOG]
    default_severity = Severity.MEDIUM
    tests = okta_login_without_push_marker_tests
    # configure this Push marker based on your environment
    PUSH_MARKER = "PS_mxzqarw"

    def rule(self, event):
        return not event.deep_get("client", "userAgent", "rawUserAgent", default="").endswith(self.PUSH_MARKER)

    def title(self, event):
        actor = event.deep_get("actor", "displayName")
        return f"{actor} logged in from device without expected Push marker"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

duo_admin_bypass_code_viewed_tests: list[RuleTest] = [
    RuleTest(
        name="Bypass View",
        expected_result=True,
        log={
            "action": "bypass_view",
            "description": '{"user_id": "D1234", "bypass_code_id": "D5678"}',
            "isotimestamp": "2022-12-14 21:17:54",
            "object": "target@example.io",
            "timestamp": "2022-12-14 21:17:54",
            "username": "Homer Simpson",
        },
    ),
    RuleTest(
        name="Bypass Create",
        expected_result=False,
        log={
            "action": "bypass_create",
            "description": '{"bypass": "", "count": 1, "valid_secs": 3600, "auto_generated": true, "remaining_uses": 1, "user_id": "D12345", "bypass_code_ids": ["A12345"]}',
            "isotimestamp": "2022-12-14 21:17:39",
            "object": "target@example.io",
            "timestamp": "2022-12-14 21:17:39",
            "username": "Homer Simpson",
        },
    ),
]


@panther_managed
class DuoAdminBypassCodeViewed(Rule):
    default_description = "An administrator viewed the MFA bypass code for a user."
    display_name = "Duo Admin Bypass Code Viewed"
    default_reference = "https://duo.com/docs/adminapi"
    default_runbook = "Confirm this behavior is authorized. The security of your Duo application is tied to the security of your secret key (skey). Secure it as you would any sensitive credential. You should not share it with unauthorized individuals or email it to anyone under any circumstances!"
    default_severity = Severity.MEDIUM
    log_types = [LogType.DUO_ADMINISTRATOR]
    id = "Duo.Admin.Bypass.Code.Viewed-prototype"
    tests = duo_admin_bypass_code_viewed_tests

    def rule(self, event):
        # Return True to match the log event and trigger an alert.
        return event.get("action", "") == "bypass_view"

    def title(self, event):
        # If no 'dedup' function is defined, the return value
        # of this method will act as deduplication string.
        return f"Duo: [{event.get('username', '<NO_USER_FOUND>')}] viewed an MFA bypass code for [{event.get('object', '<NO_OBJECT_FOUND>')}]."



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

gcp_access_attempts_violating_iap_access_controls_tests: list[RuleTest] = [
    RuleTest(
        name="Blocked By IAP",
        expected_result=True,
        log={
            "httprequest": {
                "latency": "0.048180s",
                "remoteIp": "1.2.3.4",
                "requestMethod": "GET",
                "requestSize": 77,
                "requestUrl": "http://6.7.8.9/",
                "responseSize": 211,
                "status": 403,
                "userAgent": "curl/7.85.0",
            },
            "insertid": "u94qwjf25yzns",
            "jsonpayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.loadbalancing.type.LoadBalancerLogEntry",
                "remoteIp": "1.2.3.4",
                "statusDetails": "handled_by_identity_aware_proxy",
            },
            "logname": "projects/gcp-project1/logs/requests",
            "p_any_ip_addresses": ["6.7.8.9", "1.2.3.4"],
            "p_any_trace_ids": ["projects/gcp-project1/traces/dd43c6eb7046da54fa3724d2753262e6"],
            "p_event_time": "2023-03-09 23:19:25.712",
            "p_log_type": "GCP.HTTPLoadBalancer",
            "p_parse_time": "2023-03-09 23:21:14.47",
            "p_row_id": "be93fccee09dd2f1b0b2d9ee16d5d704",
            "p_schema_version": 0,
            "p_source_id": "964c7894-9a0d-4ddf-864f-0193438221d6",
            "p_source_label": "panther-gcp-logsource",
            "receivetimestamp": "2023-03-09 23:19:26.392",
            "resource": {
                "labels": {
                    "backend_service_name": "web-backend-service",
                    "forwarding_rule_name": "http-content-rule",
                    "project_id": "gcp-project1",
                    "target_proxy_name": "http-lb-proxy-2",
                    "url_map_name": "web-map-http-2",
                    "zone": "global",
                },
                "type": "http_load_balancer",
            },
            "severity": "INFO",
            "spanid": "d75cc31c93528953",
            "timestamp": "2023-03-09 23:19:25.712",
            "trace": "projects/gcp-project1/traces/dd43c6eb7046da54fa3724d2753262e6",
        },
    ),
    RuleTest(
        name="Redirected by IAP",
        expected_result=False,
        log={
            "httprequest": {
                "latency": "0.048180s",
                "remoteIp": "1.2.3.4",
                "requestMethod": "GET",
                "requestSize": 77,
                "requestUrl": "http://6.7.8.9/",
                "responseSize": 211,
                "status": 302,
                "userAgent": "curl/7.85.0",
            },
            "insertid": "u94qwjf25yzns",
            "jsonpayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.loadbalancing.type.LoadBalancerLogEntry",
                "remoteIp": "1.2.3.4",
                "statusDetails": "handled_by_identity_aware_proxy",
            },
            "logname": "projects/gcp-project1/logs/requests",
            "p_any_ip_addresses": ["6.7.8.9", "1.2.3.4"],
            "p_any_trace_ids": ["projects/gcp-project1/traces/dd43c6eb7046da54fa3724d2753262e6"],
            "p_event_time": "2023-03-09 23:19:25.712",
            "p_log_type": "GCP.HTTPLoadBalancer",
            "p_parse_time": "2023-03-09 23:21:14.47",
            "p_row_id": "be93fccee09dd2f1b0b2d9ee16d5d704",
            "p_schema_version": 0,
            "p_source_id": "964c7894-9a0d-4ddf-864f-0193438221d6",
            "p_source_label": "panther-gcp-logsource",
            "receivetimestamp": "2023-03-09 23:19:26.392",
            "resource": {
                "labels": {
                    "backend_service_name": "web-backend-service",
                    "forwarding_rule_name": "http-content-rule",
                    "project_id": "gcp-project1",
                    "target_proxy_name": "http-lb-proxy-2",
                    "url_map_name": "web-map-http-2",
                    "zone": "global",
                },
                "type": "http_load_balancer",
            },
            "severity": "INFO",
            "spanid": "d75cc31c93528953",
            "timestamp": "2023-03-09 23:19:25.712",
            "trace": "projects/gcp-project1/traces/dd43c6eb7046da54fa3724d2753262e6",
        },
    ),
]


@panther_managed
class GCPAccessAttemptsViolatingIAPAccessControls(Rule):
    default_description = "GCP Access Attempts Violating IAP Access Controls"
    display_name = "GCP Access Attempts Violating IAP Access Controls"
    default_reference = "https://cloud.google.com/iap/docs/concepts-overview"
    default_severity = Severity.MEDIUM
    log_types = [LogType.GCP_HTTP_LOAD_BALANCER]
    id = "GCP.Access.Attempts.Violating.IAP.Access.Controls-prototype"
    tests = gcp_access_attempts_violating_iap_access_controls_tests

    def rule(self, event):
        return all(
            [
                deep_get(event, "resource", "type", default="") == "http_load_balancer",
                deep_get(event, "jsonPayload", "statusDetails", default="") == "handled_by_identity_aware_proxy",
                not any(
                    [
                        str(deep_get(event, "httprequest", "status", default=0)).startswith("2"),
                        str(deep_get(event, "httprequest", "status", default=0)).startswith("3"),
                    ],
                ),
            ],
        )

    def title(self, event):
        source = deep_get(event, "jsonPayload", "remoteIp", default="<SRC_IP_NOT_FOUND>")
        request_url = deep_get(event, "httprequest", "requestUrl", default="<REQUEST_URL_NOT_FOUND>")
        return f"GCP: Request Violating IAP controls from [{source}] to [{request_url}]"



"""Common code for lookup table helpers"""

from collections.abc import Mapping, Sequence

from pypanther.helpers.panther_base_helpers import deep_get

ENRICHMENT_KEY = "p_enrichment"
IGNORE_ENRICHMENTS = "p_any_"


# pylint: disable=too-few-public-methods
class LookupTableMatches:
    def __init__(self):
        self.lut_matches = None
        self._p_matched = {}

    def _register(self, event, lookuptable_name: str):
        self.lut_matches = deep_get(event, ENRICHMENT_KEY, lookuptable_name)

    def _lookup(self, match_field: str, *keys) -> list or str:
        match = deep_get(self.lut_matches, match_field)
        if not match:
            return None
        if isinstance(match, Sequence) and not isinstance(match, str):
            return [deep_get(match_value, *keys) if match_value else None for match_value in match]
        return deep_get(match, *keys)

    @property
    def p_matched(self):
        return self._p_matched

    def p_matches(self, event: dict, p_match: str = "") -> dict:
        """
        Collect enrichments by searching for a value match in the p_match field

        Parameters
        ----------
        event (dict): the original log event, as passed to rule(event)
        p_match (str): the value to match on in a p_match field

        Returns
        -------
        dict: All enrichments that hold the searched value in the p_match field

        """
        event = event or {}
        matched_items = {}
        for lut_name in deep_get(event, ENRICHMENT_KEY, default={}).keys():
            if lut_name.startswith(IGNORE_ENRICHMENTS):
                continue
            for en_values in deep_get(event, ENRICHMENT_KEY, lut_name, default={}).values():
                if isinstance(en_values, Sequence):
                    for val in en_values:
                        if deep_get(val, "p_match", default="") == p_match:
                            matched_items[lut_name] = val
                if isinstance(en_values, Mapping):
                    if deep_get(en_values, "p_match", default="") == p_match:
                        matched_items[lut_name] = en_values
        self._p_matched = matched_items
        return matched_items



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

signinwith_awscl_iprompt_tests: list[RuleTest] = [
    RuleTest(
        name="Test-291327",
        expected_result=True,
        log={
            "eventName": "ListApplications",
            "eventSource": "sso.amazonaws.com",
            "eventTime": "...",
            "eventVersion": "1.08",
            "sourceIPAddress": "<Victim source IP>",
            "userAgent": "<Victim browser user agent>",
            "userIdentity": {
                "accountId": "<organization master account ID>",
                "principalId": "<internal victim user id>",
                "type": "Unknown",
                "userName": "<victim display name>",
            },
        },
    ),
]


@panther_managed
class SigninwithAWSCLIprompt(Rule):
    id = "Sign-in.with.AWS.CLI.prompt-prototype"
    display_name = "SIGNAL - Sign-in with AWS CLI prompt"
    create_alert = False
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.INFO
    tests = signinwith_awscl_iprompt_tests

    def rule(self, event):
        return event.get("eventSource") == "sso.amazonaws.com" and event.get("eventName") == "ListApplications"



"""Used to define global helpers and variables."""

import json
import re
from base64 import b64decode
from binascii import Error as AsciiError
from collections import OrderedDict
from collections.abc import Mapping
from datetime import datetime
from fnmatch import fnmatch
from functools import reduce
from ipaddress import ip_address, ip_network
from typing import Any, List, Optional, Sequence, Union

from pypanther.helpers.panther_config import config

# # # # # # # # # # # # # #
#       Exceptions        #
# # # # # # # # # # # # # #


class PantherUnexpectedAlert(Exception):
    """Error returned when a Panther detection encounters an unexpected situation."""


# # # # # # # # # # # # # #
#   Compliance Helpers    #
# # # # # # # # # # # # # #

# Expects a map with a Key 'Tags' that maps to a map of key/value string pairs, or None if no
# tags are present.
# All Panther defined resources meet this requirement.
CDE_TAG_KEY = "environment"
CDE_TAG_VALUE = "pci"


# Defaults to True to assume something is in scope if it is not tagged
def in_pci_scope_tags(resource):
    if resource.get("Tags") is None:
        return True
    return resource["Tags"].get(CDE_TAG_KEY) == CDE_TAG_VALUE


PCI_NETWORKS = config.PCI_NETWORKS


# Expects a string in cidr notation (e.g. '10.0.0.0/24') indicating the ip range being checked
# Returns True if any ip in the range is marked as in scope
def is_pci_scope_cidr(ip_range):
    return any(ip_network(ip_range).overlaps(pci_network) for pci_network in PCI_NETWORKS)


DMZ_NETWORKS = config.DMZ_NETWORKS


# Expects a string in cidr notation (e.g. '10.0.0.0/24') indicating the ip range being checked
# Returns True if any ip in the range is marked as DMZ space.
def is_dmz_cidr(ip_range):
    """This function determines whether a given IP range is within the defined DMZ IP range."""
    return any(ip_network(ip_range).overlaps(dmz_network) for dmz_network in DMZ_NETWORKS)


# Defaults to False to assume something is not a DMZ if it is not tagged
def is_dmz_tags(resource, dmz_tags):
    """This function determines whether a given resource is tagged as existing in a DMZ."""
    if resource["Tags"] is None:
        return False
    for key, value in dmz_tags:
        if resource["Tags"].get(key) == value:
            return True
    return False


# Function variables here so that implementation details of these functions can be changed without
# having to rename the function in all locations its used, or having an outdated name on the actual
# function being used, etc.
IN_PCI_SCOPE = in_pci_scope_tags

# # # # # # # # # # # # # #
#      GSuite Helpers     #
# # # # # # # # # # # # # #

GSUITE_PARAMETER_VALUES = [
    "value",
    "intValue",
    "boolValue",
    "multiValue",
    "multiIntValue",
    "messageValue",
    "multiMessageValue",
]


# GSuite parameters are formatted as a list of dictionaries, where each dictionary has a 'name' key
# that maps to the name of the parameter, and one key from GSUITE_PARAMETER_VALUES that maps to the
# value of the parameter. This means to lookup the value of a particular parameter, you must
# traverse the entire list of parameters to find it and then know (or guess) what type of value it
# contains. This helper function handles that for us.
#
# Example parameters list:
# parameters = [
#   {
#       "name": "event_id",
#       "value": "abc123"
#   },
#   {
#       "name": "start_time",
#       "intValue": 63731901000
#   },
#   {
#       "name": "end_time",
#       "intValue": 63731903000
#   },
#   {
#       "name": "things",
#       "multiValue": [ "DRIVE" , "MEME"]
#   }
# ]
def gsuite_parameter_lookup(parameters, key):
    for param in parameters:
        if param["name"] != key:
            continue
        for value in GSUITE_PARAMETER_VALUES:
            if value in param:
                return param[value]
        return None
    return None


# GSuite event details are formatted as a list of dictionaries. Each entry has a 'type' and 'name'.
#
# In order to find the event details of interest, you must loop through
# the list searching for a particular type and name.
#
# This helper function handles the looping functionality that is common in many of the gsuite rules
def gsuite_details_lookup(detail_type, detail_names, event):
    for details in event.get("events", {}):
        if details.get("type") == detail_type and details.get("name") in detail_names:
            return details
    # not found, return empty dict
    return {}


# # # # # # # # # # # # # #
#      Zendesk Helpers     #
# # # # # # # # # # # # # #

# key names
ZENDESK_CHANGE_DESCRIPTION = "change_description"
ZENDESK_APP_ROLE_ASSIGNED = re.compile(
    r"(?P<app>.*) role changed from (?P<old_role>.+) to (?P<new_role>.*)",
    re.IGNORECASE,
)
ZENDESK_ROLE_ASSIGNED = re.compile(r"Role changed from (?P<old_role>.+) to (?P<new_role>[^$]+)", re.IGNORECASE)


def zendesk_get_roles(event):
    old_role = ""
    new_role = ""
    role_change = event.get(ZENDESK_CHANGE_DESCRIPTION, "")
    if "\n" in role_change:
        for app_change in role_change.split("\n"):
            matches = ZENDESK_APP_ROLE_ASSIGNED.match(app_change)
            if matches:
                if old_role:
                    old_role += " ; "
                old_role += matches.group("app") + ":" + matches.group("old_role")
                if new_role:
                    new_role += " ; "
                new_role += matches.group("app") + ":" + matches.group("new_role")
    else:
        matches = ZENDESK_ROLE_ASSIGNED.match(role_change)
        if matches:
            old_role = matches.group("old_role")
            new_role = matches.group("new_role")
    if not old_role:
        old_role = "<UNKNOWN_APP>:<UNKNOWN_ROLE>"
    if not new_role:
        new_role = "<UNKNOWN_APP>:<UNKNOWN_ROLE>"
    return old_role, new_role


# # # # # # # # # # # # # #
#      Generic Helpers    #
# # # # # # # # # # # # # #


# 'additional_details' from box logs varies by event_type.
# This helper wraps the process of extracting those details.
def box_parse_additional_details(event: dict):
    additional_details = event.get("additional_details", {})
    if isinstance(additional_details, (str, bytes)):
        try:
            return json.loads(additional_details)
        except ValueError:
            return {}
    return additional_details


def okta_alert_context(event: dict):
    """Returns common context for automation of Okta alerts"""
    return {
        "event_type": event.get("eventtype", ""),
        "severity": event.get("severity", ""),
        "actor": event.get("actor", {}),
        "client": event.get("client", {}),
        "request": event.get("request", {}),
        "outcome": event.get("outcome", {}),
        "target": event.get("target", []),
        "debug_context": event.get("debugcontext", {}),
        "authentication_context": event.get("authenticationcontext", {}),
        "security_context": event.get("securitycontext", {}),
        "ips": event.get("p_any_ip_addresses", []),
    }


def crowdstrike_detection_alert_context(event: dict):
    """Returns common context for Crowdstrike detections"""
    return {
        "aid": get_crowdstrike_field(event, "aid", default=""),
        "user": get_crowdstrike_field(event, "UserName", default=""),
        "console-link": get_crowdstrike_field(event, "FalconHostLink", default=""),
        "commandline": get_crowdstrike_field(event, "CommandLine", default=""),
        "parentcommandline": get_crowdstrike_field(event, "ParentCommandLine", default=""),
        "filename": get_crowdstrike_field(event, "FileName", default=""),
        "filepath": get_crowdstrike_field(event, "FilePath", default=""),
        "description": get_crowdstrike_field(event, "DetectDescription", default=""),
        "action": get_crowdstrike_field(event, "PatternDispositionDescription", default=""),
    }


def crowdstrike_process_alert_context(event: dict):
    """Returns common process context for Crowdstrike detections"""
    return {
        "aid": get_crowdstrike_field(event, "aid", default=""),
        "CommandLine": get_crowdstrike_field(event, "CommandLine", default=""),
        "TargetProcessId": get_crowdstrike_field(event, "TargetProcessId", default=""),
        "RawProcessId": get_crowdstrike_field(event, "RawProcessId", default=""),
        "ParentBaseFileName": get_crowdstrike_field(event, "ParentBaseFileName", default=""),
        "ParentProcessId": get_crowdstrike_field(event, "ParentProcessId", default=""),
        "ImageFileName": get_crowdstrike_field(event, "ImageFileName", default=""),
        "SHA256Hash": get_crowdstrike_field(event, "SHA256HashData", default=""),
        "platform": get_crowdstrike_field(event, "event_platform", default=""),
    }


def crowdstrike_network_detection_alert_context(event: dict):
    """Returns common network context for Crowdstrike detections"""
    return {
        "LocalAddressIP4": get_crowdstrike_field(event, "LocalAddressIP4", default=""),
        "LocalPort": get_crowdstrike_field(event, "LocalPort", default=""),
        "RemoteAddressIP4": get_crowdstrike_field(event, "RemoteAddressIP4", default=""),
        "RemotePort": get_crowdstrike_field(event, "RemotePort", default=""),
        "Protocol": get_crowdstrike_field(event, "Protocol", default=""),
        "event_simpleName": get_crowdstrike_field(event, "event_simpleName", default=""),
        "aid": get_crowdstrike_field(event, "aid", default=""),
        "ContextProcessId": get_crowdstrike_field(event, "ContextProcessId", default=""),
    }


def filter_crowdstrike_fdr_event_type(event, name: str) -> bool:
    """
    Checks if the event belongs to the Crowdstrike.FDREvent log type
    and the event type is not the name parameter.
    """
    if event.get("p_log_type") != "Crowdstrike.FDREvent":
        return False
    return event.get("fdr_event_type", "") != name


def get_crowdstrike_field(event, field_name, default=None):
    return (
        deep_get(event, field_name)
        or deep_get(event, "event", field_name)
        or deep_get(event, "unknown_payload", field_name)
        or default
    )


def slack_alert_context(event: dict):
    return {
        "actor-name": deep_get(event, "actor", "user", "name", default="<MISSING_NAME>"),
        "actor-email": deep_get(event, "actor", "user", "email", default="<MISSING_EMAIL>"),
        "actor-ip": deep_get(event, "context", "ip_address", default="<MISSING_IP>"),
        "user-agent": deep_get(event, "context", "ua", default="<MISSING_UA>"),
    }


def github_alert_context(event: dict):
    return {
        "action": event.get("action", ""),
        "actor": event.get("actor", ""),
        "actor_location": deep_get(event, "actor_location", "country_code"),
        "org": event.get("org", ""),
        "repo": event.get("repo", ""),
        "user": event.get("user", ""),
    }


def deep_get(dictionary: dict, *keys, default=None):
    """
    Safely return the value of an arbitrarily nested map

    Inspired by https://bit.ly/3a0hq9E
    """
    out = reduce(lambda d, key: d.get(key, default) if isinstance(d, Mapping) else default, keys, dictionary)
    if out is None:
        return default
    return out


# pylint: disable=too-complex,too-many-return-statements
def deep_walk(
    obj: Optional[Any],
    *keys: str,
    default: Optional[str] = None,
    return_val: str = "all",
) -> Union[Optional[Any], Optional[List[Any]]]:
    """
    Safely retrieve a value stored in complex dictionary structure

    Similar to deep_get but supports accessing dictionary keys within nested lists as well

    Parameters
    ----------
    obj (any): the original log event passed to rule(event)
               and nested objects retrieved recursively
    keys (str): comma-separated list of keys used to traverse the event object
    default (str): the default value to return if the desired key's value is not present
    return_val (str): string specifying which value to return
                      possible values are "first", "last", or "all"

    Returns
    -------
    any | list[any]: A single value if return_val is "first", "last",
                     or if "all" is a list containing one element,
                     otherwise a list of values

    """

    def _empty_list(sub_obj: Any):
        return (
            all(_empty_list(next_obj) for next_obj in sub_obj)
            if isinstance(sub_obj, Sequence) and not isinstance(sub_obj, str)
            else False
        )

    if not keys:
        return default if _empty_list(obj) else obj

    current_key = keys[0]
    found: OrderedDict = OrderedDict()

    if isinstance(obj, Mapping):
        next_key = obj.get(current_key, None)
        return (
            deep_walk(next_key, *keys[1:], default=default, return_val=return_val) if next_key is not None else default
        )
    if isinstance(obj, Sequence) and not isinstance(obj, str):
        for item in obj:
            value = deep_walk(item, *keys, default=default, return_val=return_val)
            if value is not None:
                if isinstance(value, Sequence) and not isinstance(value, str):
                    for sub_item in value:
                        found[sub_item] = None
                else:
                    found[value] = None

    found_list: list[Any] = list(found.keys())
    if not found_list:
        return default
    return {
        "first": found_list[0],
        "last": found_list[-1],
        "all": found_list[0] if len(found_list) == 1 else found_list,
    }.get(return_val, "all")


def get_val_from_list(list_of_dicts, return_field_key, field_cmp_key, field_cmp_val):
    """
    Return a specific field in a list of Python dictionaries.
    We return the empty set if the comparison key is not found
    """
    values_of_return_field = set()
    for item in list_of_dicts:
        if item.get(field_cmp_key) == field_cmp_val:
            values_of_return_field.add(item.get(return_field_key))
    return values_of_return_field


def aws_strip_role_session_id(user_identity_arn):
    # The ARN structure is arn:aws:sts::123456789012:assumed-role/RoleName/<sessionId>
    arn_parts = user_identity_arn.split("/")
    if arn_parts:
        return "/".join(arn_parts[:2])
    return user_identity_arn


def aws_rule_context(event: dict):
    return {
        "eventName": event.get("eventName", "<MISSING_EVENT_NAME>"),
        "eventSource": event.get("eventSource", "<MISSING_ACCOUNT_ID>"),
        "awsRegion": event.get("awsRegion", "<MISSING_AWS_REGION>"),
        "recipientAccountId": event.get("recipientAccountId", "<MISSING_ACCOUNT_ID>"),
        "sourceIPAddress": event.get("sourceIPAddress", "<MISSING_SOURCE_IP>"),
        "userAgent": event.get("userAgent", "<MISSING_USER_AGENT>"),
        "userIdentity": event.get("userIdentity", "<MISSING_USER_IDENTITY>"),
    }


def aws_guardduty_context(event: dict):
    return {
        "description": event.get("description", "<MISSING DESCRIPTION>"),
        "severity": event.get("severity", "<MISSING SEVERITY>"),
        "id": event.get("id", "<MISSING ID>"),
        "type": event.get("type", "<MISSING TYPE>"),
        "resource": event.get("resource", {}),
        "service": event.get("service", {}),
    }


def eks_panther_obj_ref(event: dict):
    user = deep_get(event, "user", "username", default="<NO_USERNAME>")
    source_ips = event.get("sourceIPs", ["0.0.0.0"])  # nosec
    verb = event.get("verb", "<NO_VERB>")
    obj_name = deep_get(event, "objectRef", "name", default="<NO_OBJECT_NAME>")
    obj_ns = deep_get(event, "objectRef", "namespace", default="<NO_OBJECT_NAMESPACE>")
    obj_res = deep_get(event, "objectRef", "resource", default="<NO_OBJECT_RESOURCE>")
    obj_subres = deep_get(event, "objectRef", "subresource", default="")
    p_source_label = event.get("p_source_label", "<NO_P_SOURCE_LABEL>")
    if obj_subres:
        obj_res = "/".join([obj_res, obj_subres])
    return {
        "actor": user,
        "ns": obj_ns,
        "object": obj_name,
        "resource": obj_res,
        "sourceIPs": source_ips,
        "verb": verb,
        "p_source_label": p_source_label,
    }


def is_ip_in_network(ip_addr, networks):
    """Check that a given IP is within a list of IP ranges"""
    return any(ip_address(ip_addr) in ip_network(network) for network in networks)


def pattern_match(string_to_match: str, pattern: str):
    """Wrapper around fnmatch for basic pattern globs"""
    return fnmatch(string_to_match, pattern)


def pattern_match_list(string_to_match: str, patterns: Sequence[str]):
    """Check that a string matches any pattern in a given list"""
    return any(fnmatch(string_to_match, p) for p in patterns)


def get_binding_deltas(event):
    """
    A GCP helper function to return the binding deltas from audit events

    Binding deltas provide context on a permission change, including the
    action, role, and member associated with the request.
    """
    if event.get("protoPayload", {}).get("methodName") != "SetIamPolicy":
        return []

    service_data = event.get("protoPayload", {}).get("serviceData")
    if not service_data:
        return []

    # Reference: bit.ly/2WsJdZS
    binding_deltas = service_data.get("policyDelta", {}).get("bindingDeltas")
    if not binding_deltas:
        return []
    return binding_deltas


def msft_graph_alert_context(event):
    return {
        "category": event.get("category", ""),
        "description": event.get("description", ""),
        "userStates": event.get("userStates", []),
        "fileStates": event.get("fileStates", []),
        "hostStates": event.get("hostStates", []),
    }


def m365_alert_context(event):
    return {
        "operation": event.get("Operation", ""),
        "organization_id": event.get("OrganizationId", ""),
        "client_ip": event.get("ClientIp", ""),
        "extended_properties": event.get("ExtendedProperties", []),
        "modified_properties": event.get("ModifiedProperties", []),
        "application": event.get("Application", ""),
        "actor": event.get("Actor", []),
    }


def defang_ioc(ioc):
    """Return defanged IOC from 1.1.1.1 to 1[.]1[.]1[.]1"""
    return ioc.replace(".", "[.]")


def panther_nanotime_to_python_datetime(panther_time: str) -> datetime:
    panther_time_micros = re.search(r"\.(\d+)", panther_time).group(1)
    panther_time_micros_rounded = panther_time_micros[0:6]
    panther_time_rounded = re.sub(r"\.\d+", f".{panther_time_micros_rounded}", panther_time)
    panther_time_format = r"%Y-%m-%d %H:%M:%S.%f"
    return datetime.strptime(panther_time_rounded, panther_time_format)


def golang_nanotime_to_python_datetime(golang_time: str) -> datetime:
    golang_time_format = r"%Y-%m-%dT%H:%M:%S.%fZ"
    # Golang fractional seconds include a mix of microseconds and
    # nanoseconds, which doesn't play well with Python's microseconds datetimes.
    # This rounds the fractional seconds to a microsecond-size.
    golang_time_micros = re.search(r"\.(\d+)Z", golang_time).group(1)
    golang_time_micros_rounded = golang_time_micros[0:6]
    golang_time_rounded = re.sub(r"\.\d+Z", f".{golang_time_micros_rounded}Z", golang_time)
    return datetime.strptime(golang_time_rounded, golang_time_format)


def is_base64(b64: str) -> str:
    # if the string is base64 encoded, return the decoded ASCII string
    # otherwise return an empty string
    # handle false positives for very short strings
    if len(b64) < 12:
        return ""
    # Pad args with "=" to ensure proper decoding
    b64 = b64.ljust((len(b64) + 3) // 4 * 4, "=")
    # Check if the matched string can be decoded back into ASCII
    try:
        return b64decode(b64, validate=True).decode("ascii")
    except AsciiError:
        pass
    except UnicodeDecodeError:
        pass
    return ""


def key_value_list_to_dict(list_objects: List[dict], key: str, value: str) -> dict:
    # Convert a list of dictionaries to a single dictionary
    # example: [{'key': 'a', 'value': 1}, {'key': 'b', 'value': 2}]
    # becomes: {'a': 1, 'b': 2}
    return {item[key]: item[value] for item in list_objects}



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

cisco_umbrella_dns_blocked_tests: list[RuleTest] = [
    RuleTest(
        name="Domain Blocked",
        expected_result=True,
        log={
            "action": "Blocked",
            "internalIp": "136.24.229.58",
            "externalIp": "136.24.229.58",
            "timestamp": "2020-05-21 19:20:25.000",
            "responseCode": "NOERROR",
            "domain": "malware.gvt2.com.",
        },
    ),
    RuleTest(
        name="Action Allowed",
        expected_result=False,
        log={
            "action": "Allowed",
            "internalIp": "136.24.229.58",
            "externalIp": "136.24.229.58",
            "timestamp": "2020-05-21 19:20:25.000",
            "responseCode": "NOERROR",
            "domain": "beacons3.gvt2.com.",
        },
    ),
]


@panther_managed
class CiscoUmbrellaDNSBlocked(Rule):
    id = "CiscoUmbrella.DNS.Blocked-prototype"
    display_name = "Cisco Umbrella Domain Blocked"
    dedup_period_minutes = 480
    log_types = [LogType.CISCO_UMBRELLA_DNS]
    tags = ["DNS"]
    default_severity = Severity.LOW
    default_description = "Monitor blocked domains"
    default_runbook = "Inspect the blocked domain and lookup for malware"
    default_reference = "https://support.umbrella.com/hc/en-us/articles/230563627-How-to-determine-if-a-domain-or-resource-is-being-blocked-using-Chrome-Net-Internals"
    summary_attributes = ["action", "internalIp", "externalIp", "domain", "responseCode"]
    tests = cisco_umbrella_dns_blocked_tests

    def rule(self, event):
        return event.get("action") == "Blocked"

    def title(self, event):
        return "Access denied to domain " + event.get("domain", "<UNKNOWN_DOMAIN>")



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

gcp_inbound_sso_profile_created_tests: list[RuleTest] = [
    RuleTest(
        name="InboundSsoProfileDeleted-False",
        expected_result=False,
        log={
            "insertId": "chtsf1e7iek8",
            "logName": "organizations/325169835352/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@example.com"},
                "metadata": {
                    "@type": "type.googleapis.com/ccc_hosted_reporting.ActivityProto",
                    "activityId": {"timeUsec": "1700250882598435", "uniqQualifier": "6879748717081533837"},
                    "event": [
                        {
                            "eventId": "dd0c44e2",
                            "eventName": "INBOUND_SSO_PROFILE_DELETED",
                            "eventType": "INBOUND_SSO_SETTINGS",
                            "parameter": [
                                {
                                    "label": "LABEL_OPTIONAL",
                                    "name": "INBOUND_SSO_PROFILE_NAME",
                                    "type": "TYPE_STRING",
                                    "value": "inboundSamlSsoProfiles/01lmf7wd3jt9atc",
                                },
                            ],
                        },
                    ],
                },
                "methodName": "google.admin.AdminService.inboundSsoProfileDeleted",
                "requestMetadata": {"destinationAttributes": {}, "requestAttributes": {}},
                "resourceName": "organizations/123456789012/inboundSsoSettings",
                "serviceName": "admin.googleapis.com",
            },
            "receiveTimestamp": "2023-11-17T19:54:43.367407397Z",
            "resource": {
                "labels": {
                    "method": "google.admin.AdminService.inboundSsoProfileDeleted",
                    "service": "admin.googleapis.com",
                },
                "type": "audited_resource",
            },
            "severity": "NOTICE",
            "timestamp": "2023-11-17T19:54:42.598435Z",
        },
    ),
    RuleTest(
        name="InboundSsoProfileUpdated-True",
        expected_result=True,
        log={
            "insertId": "crpr6bdcjfg",
            "logName": "organizations/123456789012/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@@example.com"},
                "metadata": {
                    "@type": "type.googleapis.com/ccc_hosted_reporting.ActivityProto",
                    "activityId": {"timeUsec": "1700250956956215", "uniqQualifier": "2009471038637356014"},
                    "event": [
                        {
                            "eventId": "bdbc47ad",
                            "eventName": "INBOUND_SSO_PROFILE_UPDATED",
                            "eventType": "INBOUND_SSO_SETTINGS",
                            "parameter": [
                                {
                                    "label": "LABEL_OPTIONAL",
                                    "name": "INBOUND_SSO_PROFILE_CHANGES",
                                    "type": "TYPE_STRING",
                                    "value": "Display Name : { oldValue: Test Profile, newValue: Test Profile Update}",
                                },
                                {
                                    "label": "LABEL_OPTIONAL",
                                    "name": "INBOUND_SSO_PROFILE_NAME",
                                    "type": "TYPE_STRING",
                                    "value": "inboundSamlSsoProfiles/03vsz0843d02br4",
                                },
                            ],
                        },
                    ],
                },
                "methodName": "google.admin.AdminService.inboundSsoProfileUpdated",
                "requestMetadata": {"destinationAttributes": {}, "requestAttributes": {}},
                "resourceName": "organizations/123456789012/inboundSsoSettings",
                "serviceName": "admin.googleapis.com",
            },
            "receiveTimestamp": "2023-11-17T19:55:57.417198068Z",
            "resource": {
                "labels": {
                    "method": "google.admin.AdminService.inboundSsoProfileUpdated",
                    "service": "admin.googleapis.com",
                },
                "type": "audited_resource",
            },
            "severity": "NOTICE",
            "timestamp": "2023-11-17T19:55:56.956215Z",
        },
    ),
    RuleTest(
        name="InboundSsoProfileCreated-True",
        expected_result=True,
        log={
            "insertId": "-rqtp5gefopij",
            "logName": "organizations/123456789012/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@example.com"},
                "metadata": {
                    "@type": "type.googleapis.com/ccc_hosted_reporting.ActivityProto",
                    "activityId": {"timeUsec": "1700250924521362", "uniqQualifier": "6051731645161637785"},
                    "event": [
                        {
                            "eventId": "637d2b33",
                            "eventName": "INBOUND_SSO_PROFILE_CREATED",
                            "eventType": "INBOUND_SSO_SETTINGS",
                            "parameter": [
                                {
                                    "label": "LABEL_OPTIONAL",
                                    "name": "INBOUND_SSO_PROFILE_NAME",
                                    "type": "TYPE_STRING",
                                    "value": "inboundSamlSsoProfiles/03vsz0843d02br4",
                                },
                            ],
                        },
                    ],
                },
                "methodName": "google.admin.AdminService.inboundSsoProfileCreated",
                "requestMetadata": {"destinationAttributes": {}, "requestAttributes": {}},
                "resourceName": "organizations/123456789012/inboundSsoSettings",
                "serviceName": "admin.googleapis.com",
            },
            "receiveTimestamp": "2023-11-17T19:55:25.012649522Z",
            "resource": {
                "labels": {
                    "method": "google.admin.AdminService.inboundSsoProfileCreated",
                    "service": "admin.googleapis.com",
                },
                "type": "audited_resource",
            },
            "severity": "NOTICE",
            "timestamp": "2023-11-17T19:55:24.521362Z",
        },
    ),
]


@panther_managed
class GCPInboundSSOProfileCreated(Rule):
    id = "GCP.Inbound.SSO.Profile.Created-prototype"
    display_name = "GCP Inbound SSO Profile Created"
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["Account Manipulation", "Additional Cloud Roles", "GCP", "Privilege Escalation"]
    reports = {"MITRE ATT&CK": ["TA0003:T1136.003", "TA0003:T1098.003", "TA0004:T1098.003"]}
    default_severity = Severity.HIGH
    default_runbook = "Ensure that the SSO profile creation or modification was expected. Adversaries may use this to persist or allow additional access or escalate their privilege.\n"
    default_reference = (
        "https://medium.com/google-cloud/detection-of-inbound-sso-persistence-techniques-in-gcp-c56f7b2a588b"
    )
    tests = gcp_inbound_sso_profile_created_tests
    METHODS = [
        "google.admin.AdminService.inboundSsoProfileCreated",
        "google.admin.AdminService.inboundSsoProfileUpdated",
    ]

    def rule(self, event):
        return event.deep_get("protoPayload", "methodName", default="") in self.METHODS

    def title(self, event):
        actor = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        event_name = event.deep_walk("protoPayload", "metadata", "event", "eventName", default="<EVENT_NAME_NOT_FOUND>")
        resource = organization_id = event.deep_walk(
            "protoPayload",
            "resourceName",
            default="<RESOURCE_NOT_FOUND>",
        ).split("/")
        organization_id = resource[resource.index("organizations") + 1]
        return f"GCP: [{actor}] performed {event_name} in organization {organization_id}"

    def alert_context(self, event):
        return {
            "resourceName": event.deep_get("protoPayload", "resourceName", default="<RESOURCE_NOT_FOUND>"),
            "serviceName": event.deep_get("protoPayload", "serviceName", default="<SERVICE_NOT_FOUND>"),
        }



from ipaddress import ip_network

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context

awss3_server_access_ip_whitelist_tests: list[RuleTest] = [
    RuleTest(
        name="Access From Approved IP",
        expected_result=False,
        log={"remoteip": "10.0.0.1", "bucket": "my-test-bucket"},
    ),
    RuleTest(
        name="Access From Unapproved IP",
        expected_result=True,
        log={"remoteip": "11.0.0.1", "bucket": "my-test-bucket"},
    ),
]


@panther_managed
class AWSS3ServerAccessIPWhitelist(Rule):
    id = "AWS.S3.ServerAccess.IPWhitelist-prototype"
    display_name = "AWS S3 Access IP Allowlist"
    enabled = False
    log_types = [LogType.AWS_S3_SERVER_ACCESS]
    tags = [
        "AWS",
        "Configuration Required",
        "Identity & Access Management",
        "Collection:Data From Cloud Storage Object",
    ]
    reports = {"MITRE ATT&CK": ["TA0009:T1530"]}
    default_severity = Severity.MEDIUM
    default_description = "Checks that the remote IP accessing the S3 bucket is in the IP allowlist.\n"
    default_runbook = "Verify whether unapproved access of S3 objects occurred, and take appropriate steps to remediate damage (for example, informing related parties of unapproved access and potentially invalidating data that was accessed). Consider updating the access policies of the S3 bucket to prevent future unapproved access.\n"
    default_reference = "https://aws.amazon.com/premiumsupport/knowledge-center/block-s3-traffic-vpc-ip/"
    summary_attributes = ["bucket", "key", "remoteip"]
    tests = awss3_server_access_ip_whitelist_tests
    # Example bucket names to watch go here
    BUCKETS_TO_MONITOR = {}
    # IP addresses (in CIDR notation) indicating approved IP ranges for accessing S3 buckets}
    ALLOWLIST_NETWORKS = {ip_network("10.0.0.0/8")}

    def rule(self, event):
        if self.BUCKETS_TO_MONITOR:
            if event.get("bucket") not in self.BUCKETS_TO_MONITOR:
                return False
        if "remoteip" not in event:
            return False
        cidr_ip = ip_network(event.get("remoteip"))
        return not any(cidr_ip.subnet_of(approved_ip_range) for approved_ip_range in self.ALLOWLIST_NETWORKS)

    def title(self, event):
        return f"Non-Approved IP access to S3 Bucket [{event.get('bucket', '<UNKNOWN_BUCKET>')}]"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

push_security_authorized_id_p_login_tests: list[RuleTest] = [
    RuleTest(
        name="Google Workspace Password Login",
        expected_result=False,
        log={
            "id": "d240e3f2-3cd6-425f-a835-dad0ff237d09",
            "new": {
                "accountId": "a93b45a7-fdce-489e-b76d-2bd6862a62ba",
                "appId": "8348ca36-d254-4e1b-8f31-6837d82fc5cb",
                "appType": "GOOGLE_WORKSPACE",
                "browser": "EDGE",
                "email": "jet.black@issp.com",
                "employeeId": "ca6cf7ce-90e6-4eb5-a262-7899bc48c39c",
                "identityProvider": "GOOGLE_WORKSPACE",
                "leakedPassword": False,
                "loginTimestamp": 1707773386.0,
                "loginType": "PASSWORD_LOGIN",
                "os": "WINDOWS",
                "passwordId": "6ae9f0b2-9300-43f0-b210-c0d3c16640f8",
                "passwordManuallyTyped": False,
                "sourceIpAddress": "35.90.103.134",
                "userAgent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36 Edg/123.0.2420.81",
                "weakPassword": False,
                "weakPasswordReasons": None,
            },
            "object": "LOGIN",
            "timestamp": 1707774319.0,
            "version": "1",
        },
    ),
    RuleTest(
        name="Microsoft 365 OIDC Login",
        expected_result=False,
        log={
            "id": "d240e3f2-3cd6-425f-a835-dad0ff237d09",
            "new": {
                "accountId": "a93b45a7-fdce-489e-b76d-2bd6862a62ba",
                "appId": "8348ca36-d254-4e1b-8f31-6837d82fc5cb",
                "appType": "DROPBOX",
                "browser": "EDGE",
                "email": "jet.black@issp.com",
                "employeeId": "ca6cf7ce-90e6-4eb5-a262-7899bc48c39c",
                "identityProvider": "MICROSOFT_365",
                "leakedPassword": False,
                "loginTimestamp": 1707773386.0,
                "loginType": "OIDC_LOGIN",
                "os": "WINDOWS",
                "passwordId": "6ae9f0b2-9300-43f0-b210-c0d3c16640f8",
                "passwordManuallyTyped": False,
                "sourceIpAddress": "35.90.103.134",
                "userAgent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36 Edg/123.0.2420.81",
                "weakPassword": False,
                "weakPasswordReasons": None,
            },
            "object": "LOGIN",
            "timestamp": 1707774319.0,
            "version": "1",
        },
    ),
    RuleTest(
        name="Okta Login",
        expected_result=True,
        log={
            "id": "d240e3f2-3cd6-425f-a835-dad0ff237d09",
            "new": {
                "accountId": "a93b45a7-fdce-489e-b76d-2bd6862a62ba",
                "appId": "8348ca36-d254-4e1b-8f31-6837d82fc5cb",
                "appType": "Dropbox",
                "browser": "EDGE",
                "email": "jet.black@issp.com",
                "employeeId": "ca6cf7ce-90e6-4eb5-a262-7899bc48c39c",
                "identityProvider": "OKTA",
                "leakedPassword": False,
                "loginTimestamp": 1707773386.0,
                "loginType": "PASSWORD_LOGIN",
                "os": "WINDOWS",
                "passwordId": "6ae9f0b2-9300-43f0-b210-c0d3c16640f8",
                "passwordManuallyTyped": False,
                "sourceIpAddress": "35.90.103.134",
                "userAgent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36 Edg/123.0.2420.81",
                "weakPassword": False,
                "weakPasswordReasons": None,
            },
            "object": "LOGIN",
            "timestamp": 1707774319.0,
            "version": "1",
        },
    ),
    RuleTest(
        name="Password Login",
        expected_result=False,
        log={
            "id": "d240e3f2-3cd6-425f-a835-dad0ff237d09",
            "new": {
                "accountId": "a93b45a7-fdce-489e-b76d-2bd6862a62ba",
                "appId": "8348ca36-d254-4e1b-8f31-6837d82fc5cb",
                "appType": "DROPBOX",
                "browser": "EDGE",
                "email": "jet.black@issp.com",
                "employeeId": "ca6cf7ce-90e6-4eb5-a262-7899bc48c39c",
                "identityProvider": None,
                "leakedPassword": False,
                "loginTimestamp": 1707773386.0,
                "loginType": "PASSWORD_LOGIN",
                "os": "WINDOWS",
                "passwordId": "6ae9f0b2-9300-43f0-b210-c0d3c16640f8",
                "passwordManuallyTyped": False,
                "sourceIpAddress": "35.90.103.134",
                "userAgent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36 Edg/123.0.2420.81",
                "weakPassword": False,
                "weakPasswordReasons": None,
            },
            "object": "LOGIN",
            "timestamp": 1707774319.0,
            "version": "1",
        },
    ),
]


@panther_managed
class PushSecurityAuthorizedIdPLogin(Rule):
    id = "Push.Security.Authorized.IdP.Login-prototype"
    display_name = "Push Security Authorized IdP Login"
    enabled = False
    create_alert = False
    log_types = [LogType.PUSH_SECURITY_ACTIVITY]
    tags = ["Configuration Required"]
    default_severity = Severity.INFO
    default_description = (
        "Login to application with unauthorized identity provider which could indicate a SAMLjacking attack."
    )
    default_reference = "https://github.com/pushsecurity/saas-attacks/blob/main/techniques/samljacking/description.md"
    inline_filters = [{"All": []}]
    tests = push_security_authorized_id_p_login_tests
    # Configure allowed identity provider logins to SaaS apps
    # "GOOGLE_WORKSPACE": {"OIDC_LOGIN", "SAML_LOGIN"},
    allowed_idps = {"OKTA": {"PASSWORD_LOGIN"}}

    def rule(self, event):
        if event.get("object") != "LOGIN":
            return False
        identity_provider = event.deep_get("new", "identityProvider")
        login_type = event.deep_get("new", "loginType")
        if identity_provider in self.allowed_idps and login_type in self.allowed_idps[identity_provider]:
            return True
        return False

    def title(self, event):
        identity_provider = event.deep_get("new", "identityProvider", default="Null identityProvider")
        login_type = event.deep_get("new", "loginType", default="Null loginType")
        app_type = event.deep_get("new", "appType", default="Null appType")
        new_email = event.deep_get("new", "email")
        return f"Authorized identity provider in use. User: {new_email}         used {identity_provider} {login_type} on {app_type}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

sentinel_one_alert_passthrough_tests: list[RuleTest] = [
    RuleTest(
        name="CRITICAL",
        expected_result=True,
        log={
            "accountid": "12345",
            "accountname": "Account1",
            "activitytype": 3608,
            "activityuuid": "f123-345-1234",
            "agentid": "1234567",
            "createdat": "2022-12-07 17:36:05.076",
            "data": {
                "accountid": "987654",
                "accountname": "Account1",
                "actoralternateid": "",
                "agentipv4": "1.2.3.4",
                "alertid": "1570395776954206544",
                "datasourcename": "SentinelOne",
                "detectedat": "2022-12-07T17:35:48Z",
                "dnsrequest": "",
                "dnsresponse": "",
                "dstip": "",
                "dstport": 0,
                "ruledescription": "test",
                "ruleid": "12345",
                "rulename": "test-rule",
                "rulescopeid": 123,
                "rulescopelevel": "E_ACCOUNT",
                "scopeid": 123,
                "scopelevel": "Group",
                "scopename": "Default Group",
                "severity": "E_CRITICAL",
                "siteid": "12345",
                "sitename": "Default site",
                "tiindicatorsource": "",
                "tiindicatortype": "",
                "tiindicatorvalue": "",
                "userid": 432134,
            },
            "groupid": "12345",
            "groupname": "Default Group",
            "id": "5423",
            "primarydescription": "Alert created for sshd from Custom Rule: test-rule in Group Default Group in Site Default site of Account Account1, detected on BobsPC.",
            "secondarydescription": "e020cd039b099b6bfdfd33d13554da5383cc4cc0",
            "siteid": "1408801957997975086",
            "sitename": "Default site",
            "updatedat": "2022-12-07 17:36:05.075",
            "userid": "1234",
        },
    ),
    RuleTest(
        name="MEDIUM",
        expected_result=True,
        log={
            "accountid": "12345",
            "accountname": "Account1",
            "activitytype": 3608,
            "activityuuid": "f123-345-1234",
            "agentid": "1234567",
            "createdat": "2022-12-07 17:36:05.076",
            "data": {
                "accountid": "987654",
                "accountname": "Account1",
                "actoralternateid": "",
                "agentipv4": "1.2.3.4",
                "alertid": "1570395776954206544",
                "datasourcename": "SentinelOne",
                "detectedat": "2022-12-07T17:35:48Z",
                "dnsrequest": "",
                "dnsresponse": "",
                "dstip": "",
                "dstport": 0,
                "ruledescription": "test",
                "ruleid": "12345",
                "rulename": "test-rule",
                "rulescopeid": 123,
                "rulescopelevel": "E_ACCOUNT",
                "scopeid": 123,
                "scopelevel": "Group",
                "scopename": "Default Group",
                "severity": "E_MEDIUM",
                "siteid": "12345",
                "sitename": "Default site",
                "tiindicatorsource": "",
                "tiindicatortype": "",
                "tiindicatorvalue": "",
                "userid": 432134,
            },
            "groupid": "12345",
            "groupname": "Default Group",
            "id": "5423",
            "primarydescription": "Alert created for sshd from Custom Rule: test-rule in Group Default Group in Site Default site of Account Account1, detected on BobsPC.",
            "secondarydescription": "e020cd039b099b6bfdfd33d13554da5383cc4cc0",
            "siteid": "1408801957997975086",
            "sitename": "Default site",
            "updatedat": "2022-12-07 17:36:05.075",
            "userid": "1234",
        },
    ),
    RuleTest(
        name="Non-Alert",
        expected_result=False,
        log={
            "accountid": "12345",
            "accountname": "Account1",
            "activitytype": 90,
            "activityuuid": "123-456-789",
            "agentid": "111111",
            "createdat": "2022-12-07 16:06:35.483",
            "data": {
                "accountname": "Account1",
                "computername": "BobsPC",
                "createdat": "2022-12-07T16:06:35.477827Z",
                "fullscopedetails": "Group Testing in Site Default site of Account Account1",
                "fullscopedetailspath": "Global / Account1 / Default site / Testing",
                "groupname": "Testing",
                "scopelevel": "Group",
                "scopename": "Testing",
                "sitename": "Default site",
                "status": "started",
            },
            "groupid": "11234",
            "groupname": "Testing",
            "id": "123564",
            "primarydescription": "Agent BobsPC started full disk scan at Wed, 07 Dec 2022, 16:06:35 UTC.",
            "siteid": "12345",
            "sitename": "Default site",
            "updatedat": "2022-12-07 16:06:35.479",
        },
    ),
]


@panther_managed
class SentinelOneAlertPassthrough(Rule):
    default_description = "SentinelOne Alert Passthrough"
    display_name = "SentinelOne Alert Passthrough"
    default_reference = "https://www.sentinelone.com/blog/feature-spotlight-introducing-the-new-threat-center/"
    default_severity = Severity.HIGH
    log_types = [LogType.SENTINELONE_ACTIVITY]
    id = "SentinelOne.Alert.Passthrough-prototype"
    tests = sentinel_one_alert_passthrough_tests
    SENTINELONE_SEVERITY = {"E_LOW": "LOW", "E_MEDIUM": "MEDIUM", "E_HIGH": "HIGH", "E_CRITICAL": "CRITICAL"}

    def rule(self, event):
        # 3608 corresponds to new alerts
        return event.get("activitytype") == 3608

    def title(self, event):
        return f"SentinelOne [{self.SENTINELONE_SEVERITY.get(deep_get(event, 'data', 'severity', default=''))}] Alert - [{deep_get(event, 'data', 'rulename')}]"

    def dedup(self, event):
        return f"s1alerts:{event.get('id')}"

    def severity(self, event):
        return self.SENTINELONE_SEVERITY.get(deep_get(event, "data", "severity", default=""), "MEDIUM")

    def alert_context(self, event):
        data_cleaned = {k: v for k, v in event.get("data", {}).items() if v != ""}
        return {
            "primarydescription": event.get("primarydescription", ""),
            "accountname": event.get("accountname", ""),
            "accountid": event.get("accountid", ""),
            "siteid": event.get("siteid", ""),
            "sitename": event.get("sitename", ""),
            "groupid": event.get("groupid", ""),
            "groupname": event.get("groupname", ""),
            "activityuuid": event.get("activityuuid", ""),
            "agentid": event.get("agentid", ""),
            "id": event.get("id", ""),
            "data": data_cleaned,
        }



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get
from pypanther.helpers.panther_default import aws_cloudtrail_success

aws_cloud_trail_iam_assume_role_blacklist_ignored_tests: list[RuleTest] = [
    RuleTest(
        name="IAM Blocklisted Role Assumed",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventID": "1111",
            "eventName": "AssumeRole",
            "eventSource": "sts.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "durationSeconds": 900,
                "roleArn": "arn:aws:iam::123456789012:role/FullAdminRole",
                "roleSessionName": "1111",
            },
            "resources": [
                {
                    "ARN": "arn:aws:iam::123456789012:role/FullAdminRole",
                    "accountId": "123456789012",
                    "type": "AWS::IAM::Role",
                },
            ],
            "responseElements": {
                "assumedRoleUser": {
                    "arn": "arn:aws:sts::123456789012:assumed-role/FullAdminRole/1111",
                    "assumedRoleId": "ABCD:1111",
                },
                "credentials": {"accessKeyId": "1111", "expiration": "Jan 01, 2019 0:00:00 PM", "sessionToken": "1111"},
            },
            "sharedEventID": "1111",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "aws-sdk-go/1.4.14 (go1.11.4; darwin; amd64)",
            "userIdentity": {
                "accesKeyId": "1111",
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:iam::123456789012:user/example-user",
                "principalId": "1111",
                "sessionContext": {"attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"}},
                "type": "IAMUser",
                "userName": "example-user",
            },
        },
    ),
    RuleTest(
        name="IAM Non Blocklisted Role Assumed",
        expected_result=False,
        log={
            "awsRegion": "us-east-1",
            "eventID": "1111",
            "eventName": "AssumeRole",
            "eventSource": "sts.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "durationSeconds": 900,
                "roleArn": "arn:aws:iam::123456789012:role/example-role",
                "roleSessionName": "1111",
            },
            "resources": [
                {
                    "ARN": "arn:aws:iam::123456789012:role/example-role",
                    "accountId": "123456789012",
                    "type": "AWS::IAM::Role",
                },
            ],
            "responseElements": {
                "assumedRoleUser": {
                    "arn": "arn:aws:sts::123456789012:assumed-role/example-role/1111",
                    "assumedRoleId": "ABCD:1111",
                },
                "credentials": {"accessKeyId": "1111", "expiration": "Jan 01, 2019 0:00:00 PM", "sessionToken": "1111"},
            },
            "sharedEventID": "1111",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "aws-sdk-go/1.4.14 (go1.11.4; darwin; amd64)",
            "userIdentity": {
                "accesKeyId": "1111",
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:iam::123456789012:user/example-user",
                "principalId": "1111",
                "sessionContext": {"attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"}},
                "type": "IAMUser",
                "userName": "example-user",
            },
        },
    ),
    RuleTest(
        name="Error Assuming IAM Blocked Role",
        expected_result=False,
        log={
            "awsRegion": "us-east-1",
            "errorCode": "ExpiredToken",
            "eventID": "1111",
            "eventName": "AssumeRole",
            "eventSource": "sts.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "durationSeconds": 900,
                "roleArn": "arn:aws:iam::123456789012:role/FullAdminRole",
                "roleSessionName": "1111",
            },
            "resources": [
                {
                    "ARN": "arn:aws:iam::123456789012:role/FullAdminRole",
                    "accountId": "123456789012",
                    "type": "AWS::IAM::Role",
                },
            ],
            "responseElements": {
                "assumedRoleUser": {
                    "arn": "arn:aws:sts::123456789012:assumed-role/FullAdminRole/1111",
                    "assumedRoleId": "ABCD:1111",
                },
                "credentials": {"accessKeyId": "1111", "expiration": "Jan 01, 2019 0:00:00 PM", "sessionToken": "1111"},
            },
            "sharedEventID": "1111",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "aws-sdk-go/1.4.14 (go1.11.4; darwin; amd64)",
            "userIdentity": {
                "accesKeyId": "1111",
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:iam::123456789012:user/example-user",
                "principalId": "1111",
                "sessionContext": {"attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"}},
                "type": "IAMUser",
                "userName": "example-user",
            },
        },
    ),
]


@panther_managed
class AWSCloudTrailIAMAssumeRoleBlacklistIgnored(Rule):
    id = "AWS.CloudTrail.IAMAssumeRoleBlacklistIgnored-prototype"
    display_name = "IAM Assume Role Blocklist Ignored"
    enabled = False
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = [
        "AWS",
        "Configuration Required",
        "Identity and Access Management",
        "Privilege Escalation:Abuse Elevation Control Mechanism",
    ]
    reports = {"MITRE ATT&CK": ["TA0004:T1548"]}
    default_severity = Severity.HIGH
    default_description = "A user assumed a role that was explicitly blocklisted for manual user assumption.\n"
    default_runbook = "Verify that this was an approved assume role action. If not, consider revoking the access immediately and updating the AssumeRolePolicyDocument to prevent this from happening again.\n"
    default_reference = "https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_boundaries.html"
    summary_attributes = ["userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    tests = aws_cloud_trail_iam_assume_role_blacklist_ignored_tests
    # This is a list of role ARNs that should not be assumed by users in normal operations
    ASSUME_ROLE_BLOCKLIST = ["arn:aws:iam::123456789012:role/FullAdminRole"]

    def rule(self, event):
        # Only considering successful AssumeRole action
        if not aws_cloudtrail_success(event) or event.get("eventName") != "AssumeRole":
            return False
        # Only considering user actions
        if deep_get(event, "userIdentity", "type") not in ["IAMUser", "FederatedUser"]:
            return False
        return deep_get(event, "requestParameters", "roleArn") in self.ASSUME_ROLE_BLOCKLIST

    def alert_context(self, event):
        return aws_rule_context(event)



import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import DataModel, DataModelMapping, LogType
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_base_helpers import gsuite_details_lookup as details_lookup


def get_event_type(event):
    # currently, only tracking a few event types
    # Pattern match this event to the recon actions
    if deep_get(event, "id", "applicationName") == "admin":
        if bool(details_lookup("DELEGATED_ADMIN_SETTINGS", ["ASSIGN_ROLE"], event)):
            return event_type.ADMIN_ROLE_ASSIGNED
    if details_lookup("login", ["login_failure"], event):
        return event_type.FAILED_LOGIN
    if deep_get(event, "id", "applicationName") == "login":
        return event_type.SUCCESSFUL_LOGIN
    return None


class StandardGSuiteReports(DataModel):
    id: str = "Standard.GSuite.Reports"
    display_name: str = "GSuite Reports"
    enabled: bool = True
    log_types: list[str] = [LogType.GSUITE_REPORTS]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="actor_user", path="$.actor.email"),
        DataModelMapping(name="assigned_admin_role", path="$.events[*].parameters[?(@.name == 'ROLE_NAME')].value"),
        DataModelMapping(name="event_type", method=get_event_type),
        DataModelMapping(name="source_ip", path="ipAddress"),
        DataModelMapping(name="user", path="$.events[*].parameters[?(@.name == 'USER_EMAIL')].value"),
    ]



from pypanther.base import DataModel, DataModelMapping, LogType


class StandardSlackAuditLogs(DataModel):
    id: str = "Standard.Slack.AuditLogs"
    display_name: str = "Slack Audit Logs"
    enabled: bool = True
    log_types: list[str] = [LogType.SLACK_AUDIT_LOGS]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="actor_user", path="$.actor.user.name"),
        DataModelMapping(name="user_agent", path="$.context.ua"),
        DataModelMapping(name="source_ip", path="$.context.ip_address"),
        DataModelMapping(name="user", path="$.entity.user.name"),
    ]



import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import DataModel, DataModelMapping, LogType


def get_event_type(event):
    # pylint: disable=too-many-return-statements
    # pylint: disable=too-many-branches
    # pylint: disable=too-complex
    if event.get("category_type") == "User":
        if event.get("action") == "Add":
            return event_type.USER_ACCOUNT_CREATED
        if event.get("action") == "Delete":
            return event_type.USER_ACCOUNT_DELETED
        if event.get("action") == "Update" and "to Admin" in event.get("operation_detail"):
            return event_type.ADMIN_ROLE_ASSIGNED
        if event.get("action") == "Update":
            return event_type.USER_ACCOUNT_MODIFIED

    if event.get("category_type") == "User Group":
        if event.get("action") == "Add":
            return event_type.USER_GROUP_CREATED
        if event.get("action") == "Update":
            return event_type.USER_GROUP_MODIFIED
        if event.get("action") == "Delete":
            return event_type.USER_GROUP_DELETED

    if event.get("category_type") == "Role":
        if event.get("action") == "Add":
            return event_type.USER_ROLE_CREATED
        if event.get("action") == "Update":
            return event_type.USER_ROLE_MODIFIED
        if event.get("action") == "Delete":
            return event_type.USER_ROLE_DELETED
    return None


class StandardZoomOperation(DataModel):
    id: str = "Standard.Zoom.Operation"
    display_name: str = None
    enabled: bool = True
    log_types: list[str] = [LogType.ZOOM_OPERATION]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="actor_user", path="operator"),
        DataModelMapping(name="event_type", method=get_event_type),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_snyk_helpers import snyk_alert_context

snyk_service_account_change_tests: list[RuleTest] = [
    RuleTest(
        name="Snyk Org Service Account ADMIN role ( CRIT )",
        expected_result=True,
        log={
            "content": {
                "role": "ADMIN",
                "rolePublicId": "d8999999-aaaa-4444-9fff-955555555555",
                "serviceAccountPublicId": "9ddddddd-4444-4111-9eee-188888888888",
            },
            "created": "2023-04-05 22:22:57.488",
            "event": "org.service_account.create",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "orgId": "21111111-a222-4eee-8ddd-a99999999999",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
    RuleTest(
        name="Snyk Group Service Account ADMIN role ( CRIT )",
        expected_result=True,
        log={
            "content": {
                "role": {
                    "created": "1999-01-22T16:02:53.848Z",
                    "description": "Admin",
                    "groupPermissions": [
                        "group.read",
                        "group.edit",
                        "group.iac.settings.read",
                        "group.iac.settings.edit",
                        "group.report.read",
                        "group.org.add",
                        "group.org.remove",
                        "group.org.list",
                        "group.user.add",
                        "group.user.read",
                        "group.user.remove",
                        "group.user.delete",
                        "group.user.role.edit",
                        "group.user.provision",
                        "group.service_account.create",
                        "group.service_account.read",
                        "group.service_account.edit",
                        "group.service_account.delete",
                        "group.audit.read",
                        "group.settings.read",
                        "group.settings.edit",
                        "group.notification_settings.read",
                        "group.notification_settings.edit",
                        "group.policy.read",
                        "group.policy.create",
                        "group.policy.edit",
                        "group.policy.delete",
                        "group.tag.read",
                        "group.tag.create",
                        "group.tag.delete",
                        "group.flags.read",
                        "group.flags.edit",
                        "group.settings.request_access.read",
                        "group.settings.request_access.edit",
                        "group.sso.read",
                        "group.sso.edit",
                        "group.sso.remove",
                        "group.roles.read",
                        "group.roles.create",
                        "group.roles.edit",
                        "group.roles.remove",
                    ],
                    "id": 1,
                    "modified": "1999-01-07T12:36:51.716Z",
                    "name": "Group Admin",
                    "orgPermissions": [
                        "org.read",
                        "org.create",
                        "org.edit",
                        "org.delete",
                        "org.report.read",
                        "org.project.create",
                        "org.project.read",
                        "org.project.edit",
                        "org.project.delete",
                        "org.project.status",
                        "org.project.test",
                        "org.project.move",
                        "org.project.integration.edit",
                        "org.project.jira.issue.read",
                        "org.project.jira.issue.create",
                        "org.project.ignore.create",
                        "org.project.ignore.read",
                        "org.project.ignore.edit",
                        "org.project.ignore.delete",
                        "org.project.pr.create",
                        "org.project.pr.skip",
                        "org.project.attributes.edit",
                        "org.project.tag.edit",
                        "org.service_account.create",
                        "org.service_account.read",
                        "org.service_account.edit",
                        "org.service_account.delete",
                        "org.user.add",
                        "org.user.invite",
                        "org.user.manage",
                        "org.user.read",
                        "org.user.delete",
                        "org.user.leave",
                        "org.user.provision",
                        "org.integration.read",
                        "org.integration.edit",
                        "org.package.test",
                        "org.billing.read",
                        "org.billing.edit",
                        "org.entitlements.read",
                        "org.audit_log.read",
                        "org.flags.read",
                        "org.flags.edit",
                        "org.outbound_webhook.read",
                        "org.outbound_webhook.create",
                        "org.outbound_webhook.delete",
                        "org.app.read",
                        "org.app.install",
                        "org.app.create",
                        "org.app.delete",
                        "org.app.edit",
                        "org.cloud_environments.read",
                        "org.cloud_environments.create",
                        "org.cloud_environments.delete",
                        "org.cloud_environments.edit",
                        "org.cloud_scans.read",
                        "org.cloud_scans.create",
                        "org.cloud_resources.read",
                        "org.cloud_artifacts.read",
                        "org.cloud_artifacts.create",
                        "org.cloud_custom_rules.read",
                        "org.cloud_custom_rules.create",
                        "org.cloud_custom_rules.edit",
                        "org.cloud_custom_rules.delete",
                    ],
                    "publicId": "89999999-feee-4333-8ccc-5aaaaaaaaaaa",
                    "role": "ADMIN",
                    "userPermissions": ["user.activate", "user.deactivate"],
                },
                "serviceAccountPublicId": "98888888-1ccc-4aaa-9fff-d66666666666",
            },
            "created": "2021-02-09 17:46:16.622",
            "event": "group.service_account.create",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
    RuleTest(
        name="Snyk Group Service Account Create - Viewer Role ( HIGH )",
        expected_result=True,
        log={
            "content": {
                "role": {
                    "created": "1999-01-22T16:02:53.848Z",
                    "description": "Viewer",
                    "groupPermissions": ["group.tag.read"],
                    "id": 3,
                    "modified": "1999-02-01T12:36:51.728Z",
                    "name": "Group Viewer",
                    "orgPermissions": ["org.read"],
                    "publicId": "24444444-addd-4ddd-a444-0ddddddddddd",
                    "role": "VIEWER",
                    "userPermissions": [],
                },
                "serviceAccountPublicId": "06666666-9eee-4ccc-8999-4bbbbbbbbbbb",
            },
            "created": "2021-01-08 17:13:01.355",
            "event": "group.service_account.create",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
    RuleTest(
        name="Snyk Service Account Edit ( MEDIUM )",
        expected_result=True,
        log={
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
            "event": "group.service_account.edit",
            "content": {
                "after": {"name": "test-SA-after"},
                "before": {"name": "test-SA-before"},
                "serviceAccountPublicId": "41111111-3fff-4eee-b111-6bbbbbbbbbbb",
            },
            "created": "2023-02-12T23:57:35.522Z",
        },
    ),
    RuleTest(
        name="Snyk Group SSO Membership sync",
        expected_result=False,
        log={
            "content": {},
            "created": "2023-03-15 13:13:13.133",
            "event": "group.sso.membership.sync",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
        },
    ),
]


@panther_managed
class SnykServiceAccountChange(Rule):
    id = "Snyk.ServiceAccount.Change-prototype"
    display_name = "Snyk Service Account Change"
    log_types = [LogType.SNYK_GROUP_AUDIT, LogType.SNYK_ORG_AUDIT]
    tags = ["Snyk"]
    default_severity = Severity.HIGH
    default_description = "Detects when Snyk Service Accounts are changed\n"
    default_runbook = "These actions in the Snyk Audit logs indicate that a ServiceAccount has been created/deleted/modified.\nService Accounts are system user accounts with an API token associated to it in place of standard user credentials. All events where the Service Account's role is ADMIN have CRITICAL severity Deletion events are marked with HIGH severity Creation events are marked with HIGH severity Edit events are marked with MEDIUM Severity\n"
    default_reference = "https://docs.snyk.io/snyk-admin/service-accounts"
    summary_attributes = ["event"]
    tests = snyk_service_account_change_tests
    ACTIONS = [
        "group.service_account.create",
        "group.service_account.delete",
        "group.service_account.edit",
        "org.service_account.create",
        "org.service_account.delete",
        "org.service_account.edit",
        "org.service_account.membership.upsert",
    ]

    def rule(self, event):
        action = deep_get(event, "event", default="<NO_EVENT>")
        return action in self.ACTIONS

    def title(self, event):
        group_or_org = "<GROUP_OR_ORG>"
        crud_operation = "<NO_OPERATION>"
        action = deep_get(event, "event", default="<NO_EVENT>")
        if "." in action:
            group_or_org = action.split(".")[0].title()
            crud_operation = action.split(".")[-1].title()
        return f"Snyk: [{group_or_org}] Service Account [{crud_operation}] performed by [{deep_get(event, 'userId', default='<NO_USERID>')}]"

    def alert_context(self, event):
        a_c = snyk_alert_context(event)
        role = deep_get(event, "content", "role", "role", default=None)
        if not role:
            role = deep_get(event, "content", "role", default=None)
        if role:
            a_c["role_permission"] = role
        return a_c

    def dedup(self, event):
        return f"{deep_get(event, 'userId', default='<NO_USERID>')}{deep_get(event, 'orgId', default='<NO_ORGID>')}{deep_get(event, 'groupId', default='<NO_GROUPID>')}{deep_get(event, 'event', default='<NO_EVENT>')}"

    def severity(self, event):
        action = deep_get(event, "event", default="<NO_EVENT>")
        role = deep_get(event, "content", "role", "role", default=None)
        if not role:
            role = deep_get(event, "content", "role", default=None)
        if all([role == "ADMIN", action.endswith((".service_account.create", ".service_account.delete"))]):
            return "CRITICAL"
        if action.endswith((".service_account.create", ".service_account.delete")):
            return "HIGH"
        return "MEDIUM"



import ipaddress

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

osquery_linux_login_from_non_office_tests: list[RuleTest] = [
    RuleTest(
        name="Non-office network login (logged_in_users)",
        expected_result=True,
        log={
            "name": "pack/incident_response/logged_in_users",
            "action": "added",
            "columns": {"host": "10.0.3.1", "type": "user", "user": "ubuntu"},
        },
    ),
    RuleTest(
        name="Non-office network login (last)",
        expected_result=True,
        log={
            "name": "pack-incident_response-last",
            "action": "added",
            "columns": {
                "host": "10.0.3.1",
                "type": "8",
                "username": "ubuntu",
                "tty": "ttys008",
                "pid": "648",
                "time": "1587502574",
            },
        },
    ),
    RuleTest(
        name="Office network login",
        expected_result=False,
        log={"name": "pack-logged_in_users", "action": "added", "columns": {"host": "192.168.1.200", "user": "ubuntu"}},
    ),
]


@panther_managed
class OsqueryLinuxLoginFromNonOffice(Rule):
    id = "Osquery.Linux.LoginFromNonOffice-prototype"
    display_name = "A Login from Outside the Corporate Office"
    enabled = False
    log_types = [LogType.OSQUERY_DIFFERENTIAL]
    tags = ["Configuration Required", "Osquery", "Linux", "Initial Access:Valid Accounts"]
    reports = {"MITRE ATT&CK": ["TA0001:T1078"]}
    default_severity = Severity.HIGH
    default_description = "A system has been logged into from a non approved IP space."
    default_runbook = "Analyze the host IP, and if possible, update allowlist or fix ACL."
    default_reference = "https://attack.mitre.org/techniques/T1078/"
    summary_attributes = ["name", "action", "p_any_ip_addresses", "p_any_domain_names"]
    tests = osquery_linux_login_from_non_office_tests
    # This is only an example network, but you can set it to whatever you'd like
    OFFICE_NETWORKS = [ipaddress.ip_network("192.168.1.100/32"), ipaddress.ip_network("192.168.1.200/32")]

    def _login_from_non_office_network(self, host):
        host_ipaddr = ipaddress.IPv4Address(host)
        non_office_logins = []
        for office_network in self.OFFICE_NETWORKS:
            non_office_logins.append(host_ipaddr in office_network)
        return not any(non_office_logins)

    def rule(self, event):
        if event.get("action") != "added":
            return False
        if "logged_in_users" in event.get("name"):
            # Only pay attention to users and not system-level accounts
            if deep_get(event, "columns", "type") != "user":
                return False
        elif "last" in event.get("name"):
            pass
        else:
            # A query we don't care about
            return False
        host_ip = deep_get(event, "columns", "host")
        return self._login_from_non_office_network(host_ip)

    def title(self, event):
        user = deep_get(event, "columns", "user", default=deep_get(event, "columns", "username"))
        return f"User [{(user if user else '<UNKNOWN_USER>')} has logged into production from a non-office network"



import ipaddress

import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import DataModel, DataModelMapping, LogType
from pypanther.helpers.panther_base_helpers import deep_get


def get_event_type(event):
    # currently, only tracking a few event types
    if event.get("eventName") == "ConsoleLogin" and deep_get(event, "userIdentity", "type") == "IAMUser":
        if deep_get(event, "responseElements", "ConsoleLogin") == "Failure":
            return event_type.FAILED_LOGIN
        if deep_get(event, "responseElements", "ConsoleLogin") == "Success":
            return event_type.SUCCESSFUL_LOGIN
    if event.get("eventName") == "CreateUser":
        return event_type.USER_ACCOUNT_CREATED
    if event.get("eventName") == "CreateAccountResult":
        return event_type.ACCOUNT_CREATED
    return None


def load_ip_address(event):
    """
    CloudTrail occasionally sets non-IPs in the sourceIPAddress field.
    This method ensures that either an IPv4 or IPv6 address is always returned.
    """
    source_ip = event.get("sourceIPAddress")
    if not source_ip:
        return None
    try:
        ipaddress.IPv4Address(source_ip)
    except ipaddress.AddressValueError:
        try:
            ipaddress.IPv6Address(source_ip)
        except ipaddress.AddressValueError:
            return None
    return source_ip


class StandardAWSCloudTrail(DataModel):
    id: str = "Standard.AWS.CloudTrail"
    display_name: str = "AWS CloudTrail"
    enabled: bool = True
    log_types: list[str] = [LogType.AWS_CLOUDTRAIL]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="actor_user", path="$.userIdentity..userName"),
        DataModelMapping(name="event_type", method=get_event_type),
        DataModelMapping(name="source_ip", method=load_ip_address),
        DataModelMapping(name="user_agent", path="userAgent"),
        DataModelMapping(name="user", path="$.responseElements.user.userName"),
        DataModelMapping(name="user_account_id", path="$.responseElements.user.userId"),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

zoom_sign_in_method_modified_tests: list[RuleTest] = [
    RuleTest(
        name="Google",
        expected_result=True,
        log={
            "action": "Update",
            "category_type": "Account",
            "operation_detail": "Sign-in Methods  - Allow users to sign in with Google: from Off to On",
            "operator": "example@example.io",
            "time": "2022-12-16 18:20:07",
        },
    ),
    RuleTest(
        name="Apple ID",
        expected_result=True,
        log={
            "action": "Update",
            "category_type": "Account",
            "operation_detail": "Sign-in Methods  - Allow users to sign in with Apple ID: from Off to On",
            "operator": "example@example.io",
            "time": "2022-12-16 18:19:57",
        },
    ),
    RuleTest(
        name="Automatic Sign Out Disabled",
        expected_result=False,
        log={
            "action": "Update",
            "category_type": "Account",
            "operation_detail": "Security  - Automatically sign users out after a specified time: from On to Off",
            "operator": "example@example.io",
            "time": "2022-12-16 18:20:42",
        },
    ),
]


@panther_managed
class ZoomSignInMethodModified(Rule):
    default_description = "A Zoom User modified your organizations sign in method."
    display_name = "Zoom Sign In Method Modified"
    default_runbook = (
        "Confirm this user acted with valid business intent and determine whether this activity was authorized."
    )
    default_reference = "https://support.zoom.com/hc/en/article?id=zm_kb&sysparm_article=KB0067602#:~:text=Go%20to%20the%20Zoom%20site,click%20Link%20and%20Sign%20In"
    default_severity = Severity.MEDIUM
    log_types = [LogType.ZOOM_OPERATION]
    id = "Zoom.Sign.In.Method.Modified-prototype"
    tests = zoom_sign_in_method_modified_tests

    def rule(self, event):
        operation_detail = event.get("operation_detail", "<NO_OPS_DETAIL>")
        operation_flag = "Sign-in Methods  - Allow users to sign in with "
        setting_flag = "from Off to On"
        return all(
            [
                event.get("action", "<NO_ACTION>") == "Update",
                event.get("category_type", "<NO_CATEGORY_TYPE>") == "Account",
                operation_detail.startswith(operation_flag),
                operation_detail.endswith(setting_flag),
            ],
        )

    def title(self, event):
        # string manipulation to grab service that allows sign-in from the operation detail
        # and clean it up a bit
        service_detail = ""
        operation_detail = event.get("operation_detail", "<NO_OPS_DETAIL>")
        operation_flag = "Sign-in Methods  - Allow users to sign in with "
        setting_flag = "from Off to On"
        if operation_detail.startswith(operation_flag) and operation_detail.endswith(setting_flag):
            service_detail = event.get("operation_detail", "<NO_OPS_DETAIL>").split("with")[1].split(":")[0].strip()
        return f"Zoom User [{event.get('operator', '<NO_OPERATOR>')}] modified your organization's sign in methods to allow users to sign in with [{service_detail}]."



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

teleport_lock_created_tests: list[RuleTest] = [
    RuleTest(
        name="A Lock was created",
        expected_result=True,
        log={
            "cluster_name": "teleport.example.com",
            "code": "TLK00I",
            "ei": 0,
            "event": "lock.created",
            "expires": "0001-01-01T00:00:00Z",
            "name": "88888888-4444-4444-4444-222222222222",
            "target": {"user": "user-to-disable"},
            "time": "2023-09-21T00:00:00.000000Z",
            "uid": "88888888-4444-4444-4444-222222222222",
            "updated_by": "max.mustermann@example.com",
            "user": "max.mustermann@example.com",
        },
    ),
]


@panther_managed
class TeleportLockCreated(Rule):
    id = "Teleport.LockCreated-prototype"
    display_name = "A Teleport Lock was created"
    log_types = [LogType.GRAVITATIONAL_TELEPORT_AUDIT]
    tags = ["Teleport"]
    default_severity = Severity.INFO
    default_description = "A Teleport Lock was created"
    default_reference = "https://goteleport.com/docs/management/admin/"
    default_runbook = "A Teleport Lock was created; this is an unusual administrative action. Investigate to understand why a Lock was created.\n"
    summary_attributes = ["event", "code", "time", "identity"]
    tests = teleport_lock_created_tests

    def rule(self, event):
        return event.get("event") == "lock.created"

    def title(self, event):
        return f"A Teleport Lock was created by {event.get('updated_by', '<UNKNOWN_UPDATED_BY>')} to Lock out user {event.get('target', {}).get('user', '<UNKNOWN_USER>')} on [{event.get('cluster_name', '<UNKNOWN_CLUSTER>')}]"



from pypanther.base import DataModel, DataModelMapping, LogType


class StandardSlackAccessLogs(DataModel):
    id: str = "Standard.Slack.AccessLogs"
    display_name: str = "Slack Access Logs"
    enabled: bool = True
    log_types: list[str] = [LogType.SLACK_ACCESS_LOGS]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="source_ip", path="ip"),
        DataModelMapping(name="user_agent", path="user_agent"),
        DataModelMapping(name="actor_user", path="username"),
    ]



from json import loads

import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_default import lookup_aws_account_name
from pypanther.helpers.panther_ipinfo_helpers import PantherIPInfoException, geoinfo_from_ip
from pypanther.helpers.panther_oss_helpers import add_parse_delay

standard_brute_force_by_ip_tests: list[RuleTest] = [
    RuleTest(
        name="AWS.CloudTrail - Successful Login",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:user/tester",
                "accountId": "123456789012",
                "userName": "testuser",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "signin.amazonaws.com",
            "eventName": "ConsoleLogin",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": None,
            "responseElements": {"ConsoleLogin": "Success"},
            "additionalEventData": {
                "LoginTo": "https://console.aws.amazon.com/console/",
                "MobileVersion": "No",
                "MFAUsed": "No",
            },
            "eventID": "1",
            "eventType": "AwsConsoleSignIn",
            "recipientAccountId": "123456789012",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2021-06-04 10:02:33.650807",
            "p_event_time": "2021-06-04 09:59:53.650807",
        },
    ),
    RuleTest(
        name="AWS.CloudTrail - Failed Login",
        expected_result=True,
        mocks=[
            RuleMock(
                object_name="geoinfo_from_ip",
                return_value='{ "ip": "111.111.111.111", "region": "UnitTestRegion", "city": "UnitTestCityNew", "country": "UnitTestCountry", "hostname": "somedomain.com", "org": "Some Org" }',
            ),
        ],
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:user/tester",
                "accountId": "123456789012",
                "userName": "tester",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "signin.amazonaws.com",
            "eventName": "ConsoleLogin",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": None,
            "responseElements": {"ConsoleLogin": "Failure"},
            "additionalEventData": {
                "LoginTo": "https://console.aws.amazon.com/console/",
                "MobileVersion": "No",
                "MFAUsed": "No",
            },
            "eventID": "1",
            "eventType": "AwsConsoleSignIn",
            "recipientAccountId": "123456789012",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2021-06-04 10:02:33.650807",
            "p_event_time": "2021-06-04 09:59:53.650807",
        },
    ),
    RuleTest(
        name="Box - Regular Event",
        expected_result=False,
        log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {"id": "12345678", "type": "user", "login": "cat@example", "name": "Bob Cat"},
            "ip_address": "111.111.111.111",
            "event_type": "DELETE",
            "p_log_type": "Box.Event",
            "p_parse_time": "2021-06-04 10:02:33.650807",
            "p_event_time": "2021-06-04 09:59:53.650807",
        },
    ),
    RuleTest(
        name="Box - Login Failed",
        expected_result=True,
        mocks=[
            RuleMock(
                object_name="geoinfo_from_ip",
                return_value='{ "ip": "111.111.111.111", "region": "UnitTestRegion", "city": "UnitTestCityNew", "country": "UnitTestCountry", "hostname": "somedomain.com", "org": "Some Org" }',
            ),
        ],
        log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {"id": "12345678", "type": "user", "login": "cat@example", "name": "Bob Cat"},
            "event_type": "FAILED_LOGIN",
            "source": {"id": "12345678", "type": "user", "name": "Bob Cat"},
            "ip_address": "111.111.111.111",
            "p_log_type": "Box.Event",
            "p_parse_time": "2021-06-04 10:02:33.650807",
            "p_event_time": "2021-06-04 09:59:53.650807",
        },
    ),
    RuleTest(
        name="GSuite - Normal Login Event",
        expected_result=False,
        log={
            "id": {"applicationName": "login"},
            "ipAddress": "111.111.111.111",
            "events": [{"type": "login", "name": "login_success"}],
            "p_log_type": "GSuite.Reports",
            "p_parse_time": "2021-06-04 10:02:33.650807",
            "p_event_time": "2021-06-04 09:59:53.650807",
        },
    ),
    RuleTest(
        name="GSuite - Failed Login Event",
        expected_result=True,
        mocks=[
            RuleMock(
                object_name="geoinfo_from_ip",
                return_value='{ "ip": "111.111.111.111", "region": "UnitTestRegion", "city": "UnitTestCityNew", "country": "UnitTestCountry", "hostname": "somedomain.com", "org": "Some Org" }',
            ),
        ],
        log={
            "actor": {"email": "bob@example.com"},
            "id": {"applicationName": "login"},
            "ipAddress": "111.111.111.111",
            "events": [{"type": "login", "name": "login_failure"}],
            "p_log_type": "GSuite.Reports",
            "p_parse_time": "2021-06-04 10:02:33.650807",
            "p_event_time": "2021-06-04 09:59:53.650807",
        },
    ),
    RuleTest(
        name="Okta - Successful Login",
        expected_result=False,
        log={
            "actor": {"alternateId": "admin", "displayName": "unknown", "id": "unknown", "type": "User"},
            "client": {"ipAddress": "111.111.111.111"},
            "eventType": "user.session.start",
            "outcome": {"reason": "VERIFICATION_ERROR", "result": "SUCCESS"},
            "p_log_type": "Okta.SystemLog",
            "p_parse_time": "2021-06-04 10:02:33.650807",
            "p_event_time": "2021-06-04 09:59:53.650807",
        },
    ),
    RuleTest(
        name="Okta - Failed Login",
        expected_result=True,
        mocks=[
            RuleMock(
                object_name="geoinfo_from_ip",
                return_value='{ "ip": "111.111.111.111", "region": "UnitTestRegion", "city": "UnitTestCityNew", "country": "UnitTestCountry", "hostname": "somedomain.com", "org": "Some Org" }',
            ),
        ],
        log={
            "actor": {"alternateId": "admin", "displayName": "unknown", "id": "unknown", "type": "User"},
            "client": {"ipAddress": "111.111.111.111"},
            "eventType": "user.session.start",
            "outcome": {"reason": "VERIFICATION_ERROR", "result": "FAILURE"},
            "p_log_type": "Okta.SystemLog",
            "p_parse_time": "2021-06-04 10:02:33.650807",
            "p_event_time": "2021-06-04 09:59:53.650807",
        },
    ),
    RuleTest(
        name="OneLogin - Normal Login Event",
        expected_result=False,
        log={
            "event_type_id": 8,
            "actor_user_id": 123456,
            "actor_user_name": "Bob Cat",
            "user_id": 123456,
            "user_name": "Bob Cat",
            "ipaddr": "111.111.111.111",
            "p_log_type": "OneLogin.Events",
            "p_parse_time": "2021-06-04 10:02:33.650807",
            "p_event_time": "2021-06-04 09:59:53.650807",
        },
    ),
    RuleTest(
        name="OneLogin - Failed Login Event",
        expected_result=True,
        mocks=[
            RuleMock(
                object_name="geoinfo_from_ip",
                return_value='{ "ip": "111.111.111.111", "region": "UnitTestRegion", "city": "UnitTestCityNew", "country": "UnitTestCountry", "hostname": "somedomain.com", "org": "Some Org" }',
            ),
        ],
        log={
            "event_type_id": 6,
            "actor_user_id": 123456,
            "actor_user_name": "Bob Cat",
            "user_id": 123456,
            "user_name": "Bob Cat",
            "ipaddr": "1.2.3.4",
            "p_log_type": "OneLogin.Events",
            "p_parse_time": "2021-06-04 10:02:33.650807",
            "p_event_time": "2021-06-04 09:59:53.650807",
        },
    ),
    RuleTest(
        name="GCP - Non Login Event",
        expected_result=False,
        log={
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "serviceName": "cloudresourcemanager.googleapis.com",
                "methodName": "SetIamPolicy",
                "authenticationInfo": {"principalEmail": "bob@example.com"},
                "requestMetadata": {"callerIP": "111.111.111.111"},
                "serviceData": {
                    "@type": "type.googleapis.com/google.iam.v1.logging.AuditData",
                    "policyDelta": {
                        "bindingDeltas": [
                            {
                                "action": "ADD",
                                "member": "cat@example.com",
                                "role": "roles/resourcemanager.organizationAdmin",
                            },
                        ],
                    },
                },
            },
            "p_log_type": "GCP.AuditLog",
            "p_parse_time": "2021-06-04 10:02:33.650807",
            "p_event_time": "2021-06-04 09:59:53.650807",
        },
    ),
    RuleTest(
        name="Asana - Failed Login",
        expected_result=True,
        mocks=[
            RuleMock(
                object_name="geoinfo_from_ip",
                return_value='{ "ip": "111.111.111.111", "region": "UnitTestRegion", "city": "UnitTestCityNew", "country": "UnitTestCountry", "hostname": "somedomain.com", "org": "Some Org" }',
            ),
        ],
        log={
            "actor": {"actor_type": "user", "email": "homer@springfield.com", "gid": "2222222", "name": "Homer"},
            "context": {"client_ip_address": "8.8.8.8", "context_type": "web"},
            "created_at": "2021-10-21T23:38:10.364Z",
            "details": {"method": ["ONE_TIME_KEY"]},
            "event_category": "logins",
            "event_type": "user_login_failed",
            "gid": "222222222",
            "resource": {"email": "homer@springfield.com", "gid": "2222222", "name": "homer", "resource_type": "user"},
            "p_log_type": "Asana.Audit",
            "p_parse_time": "2021-06-04 10:02:33.650807",
            "p_event_time": "2021-06-04 09:59:53.650807",
        },
    ),
    RuleTest(
        name="Asana - Normal Login",
        expected_result=False,
        log={
            "actor": {"actor_type": "user", "email": "homer@springfield.com", "gid": "2222222", "name": "Homer"},
            "context": {"client_ip_address": "8.8.8.8", "context_type": "web"},
            "created_at": "2021-10-21T23:38:10.364Z",
            "details": {"method": ["ONE_TIME_KEY"]},
            "event_category": "logins",
            "event_type": "user_login_succeeded",
            "gid": "222222222",
            "resource": {"email": "homer@springfield.com", "gid": "2222222", "name": "homer", "resource_type": "user"},
            "p_log_type": "Asana.Audit",
            "p_parse_time": "2021-06-04 10:02:33.650807",
            "p_event_time": "2021-06-04 09:59:53.650807",
        },
    ),
    RuleTest(
        name="1Password - Regular Login",
        expected_result=False,
        log={
            "uuid": "1234",
            "session_uuid": "5678",
            "timestamp": "2021-12-03 19:52:52",
            "category": "success",
            "type": "credentials_ok",
            "country": "US",
            "target_user": {"email": "homer@springfield.gov", "name": "Homer Simpson", "uuid": "1234"},
            "client": {
                "app_name": "1Password Browser Extension",
                "app_version": "20184",
                "ip_address": "1.1.1.1",
                "os_name": "Solaris",
                "os_version": "10",
                "platform_name": "Chrome",
                "platform_version": "96.0.4664.55",
            },
            "p_log_type": "OnePassword.SignInAttempt",
        },
    ),
    RuleTest(
        name="1Password - Failed Login",
        expected_result=True,
        mocks=[
            RuleMock(
                object_name="geoinfo_from_ip",
                return_value='{ "ip": "111.111.111.111", "region": "UnitTestRegion", "city": "UnitTestCityNew", "country": "UnitTestCountry", "hostname": "somedomain.com", "org": "Some Org" }',
            ),
        ],
        log={
            "uuid": "1234",
            "session_uuid": "5678",
            "timestamp": "2021-12-03 19:52:52",
            "category": "credentials_failed",
            "type": "password_secret_bad",
            "country": "US",
            "target_user": {"email": "homer@springfield.gov", "name": "Homer Simpson", "uuid": "1234"},
            "client": {
                "app_name": "1Password Browser Extension",
                "app_version": "20184",
                "ip_address": "111.111.111.111",
                "os_name": "Solaris",
                "os_version": "10",
                "platform_name": "Chrome",
                "platform_version": "96.0.4664.55",
            },
            "p_log_type": "OnePassword.SignInAttempt",
            "p_parse_time": "2021-06-04 10:02:33.650807",
            "p_event_time": "2021-06-04 09:59:53.650807",
        },
    ),
]


@panther_managed
class StandardBruteForceByIP(Rule):
    id = "Standard.BruteForceByIP-prototype"
    display_name = "Brute Force By IP"
    log_types = [
        LogType.ASANA_AUDIT,
        LogType.ATLASSIAN_AUDIT,
        LogType.AWS_CLOUDTRAIL,
        LogType.BOX_EVENT,
        LogType.GSUITE_REPORTS,
        LogType.OKTA_SYSTEM_LOG,
        LogType.ONELOGIN_EVENTS,
        LogType.ONEPASSWORD_SIGNIN_ATTEMPT,
    ]
    default_severity = Severity.INFO
    tags = ["DataModel", "Credential Access:Brute Force"]
    threshold = 20
    reports = {"MITRE ATT&CK": ["TA0006:T1110"]}
    default_description = "An actor user was denied login access more times than the configured threshold."
    default_runbook = "Analyze the IP they came from, and other actions taken before/after. Check if a user from this ip eventually authenticated successfully."
    default_reference = "https://owasp.org/www-community/controls/Blocking_Brute_Force_Attacks"
    summary_attributes = ["p_any_ip_addresses"]
    tests = standard_brute_force_by_ip_tests

    def rule(self, event):
        # filter events on unified data model field
        return event.udm("event_type") == event_type.FAILED_LOGIN

    def title(self, event):
        # use unified data model field in title
        log_type = event.get("p_log_type")
        title_str = f"{log_type}: User [{event.udm('actor_user')}] has exceeded the failed logins threshold"
        if log_type == "AWS.CloudTrail":
            title_str += f" in [{lookup_aws_account_name(event.get('recipientAccountId'))}]"
        return title_str

    def alert_context(self, event):
        try:
            geoinfo = geoinfo_from_ip(event=event, match_field=event.udm_path("source_ip"))
        except PantherIPInfoException:
            geoinfo = {}
        if isinstance(geoinfo, str):
            geoinfo = loads(geoinfo)
        context = {}
        context["geolocation"] = f"{geoinfo.get('city')}, {geoinfo.get('region')} in {geoinfo.get('country')}"
        context["ip"] = geoinfo.get("ip")
        context["reverse_lookup"] = geoinfo.get("hostname", "No reverse lookup hostname")
        context["ip_org"] = geoinfo.get("org", "No organization listed")
        try:
            context = add_parse_delay(event, context)
        except TypeError:
            pass
        except AttributeError:
            pass
        return context



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

google_workspace_apps_new_mobile_app_installed_tests: list[RuleTest] = [
    RuleTest(
        name="Android Calculator",
        expected_result=True,
        log={
            "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "12345"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-10 22:55:38.478000000",
                "uniqueQualifier": "12345",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "ADD_MOBILE_APPLICATION_TO_WHITELIST",
            "parameters": {
                "DEVICE_TYPE": "Android",
                "DISTRIBUTION_ENTITY_NAME": "/",
                "DISTRIBUTION_ENTITY_TYPE": "ORG_UNIT",
                "MOBILE_APP_PACKAGE_ID": "com.google.android.calculator",
            },
            "type": "MOBILE_SETTINGS",
        },
    ),
    RuleTest(
        name="Enable User Enrollement",
        expected_result=False,
        log={
            "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "12345"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-11 01:35:29.906000000",
                "uniqueQualifier": "-12345",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "CREATE_APPLICATION_SETTING",
            "parameters": {
                "APPLICATION_EDITION": "standard",
                "APPLICATION_NAME": "Security",
                "NEW_VALUE": "true",
                "ORG_UNIT_NAME": "Example IO",
                "SETTING_NAME": "Advanced Protection Program Settings - Enable user enrollment",
            },
            "type": "APPLICATION_SETTINGS",
        },
    ),
    RuleTest(
        name="ListObject Type",
        expected_result=False,
        log={
            "actor": {"email": "user@example.io", "profileId": "118111111111111111111"},
            "id": {
                "applicationName": "drive",
                "customerId": "D12345",
                "time": "2022-12-20 17:27:47.080000000",
                "uniqueQualifier": "-7312729053723258069",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "rename",
            "parameters": {
                "actor_is_collaborator_account": None,
                "billable": True,
                "doc_id": "1GGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGG",
                "doc_title": "Document Title- Found Here",
                "doc_type": "presentation",
                "is_encrypted": None,
                "new_value": ["Document Title- Found Here"],
                "old_value": ["Document Title- Old"],
                "owner": "user@example.io",
                "owner_is_shared_drive": None,
                "owner_is_team_drive": None,
                "primary_event": True,
                "visibility": "private",
            },
            "type": "access",
        },
    ),
]


@panther_managed
class GoogleWorkspaceAppsNewMobileAppInstalled(Rule):
    default_description = (
        "A new mobile application was added to your organization's mobile apps whitelist in Google Workspace Apps."
    )
    display_name = "Google Workspace Apps New Mobile App Installed"
    default_runbook = "https://admin.google.com/ac/apps/unified"
    default_reference = "https://support.google.com/a/answer/6089179?hl=en"
    default_severity = Severity.MEDIUM
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    id = "Google.Workspace.Apps.New.Mobile.App.Installed-prototype"
    tests = google_workspace_apps_new_mobile_app_installed_tests

    def rule(self, event):
        # Return True to match the log event and trigger an alert.
        return event.get("name", "") == "ADD_MOBILE_APPLICATION_TO_WHITELIST"

    def title(self, event):
        # If no 'dedup' function is defined, the return value of
        # this method will act as deduplication string.
        mobile_app_pkg_id = event.get("parameters", {}).get("MOBILE_APP_PACKAGE_ID", "<NO_MOBILE_APP_PACKAGE_ID_FOUND>")
        return f"Google Workspace User [{event.get('actor', {}).get('email', '<NO_EMAIL_FOUND>')}] added application [{mobile_app_pkg_id}] to your org's mobile application allowlist for [{event.get('parameters', {}).get('DEVICE_TYPE', '<NO_DEVICE_TYPE_FOUND>')}]."



"""Overridden Configuration values for Panther"""

"""
Here, you can override the default, example configuration values from `panther_config_defaults`

Any attribute found to be defined in here will take precedence at lookup time.
For example, we can totally re-define a value:

# Total Override
panther_config_defaults.py
```
SUSPICIOUS_DOMAINS = [ "evil.example.com" ]
```

panther_config_overrides.py
```
SUSPICIOUS_DOMAINS = [ "betrug.example.com" ]
```

and at lookup-time:
```
from pypanther.helpers.panther_config import config
print(config.SUSPICIOUS_DOMAINS)
```
prints ["betrug.example.com"]

# Mixing Values
panther_config_defaults.py
```
INTERNAL_NETWORKS = [ "10.0.0.0/8" ]
```

panther_config_overrides.py
```
import pypanther.helpers.panther_config_defaults as panther_config_defaults
INTERNAL_NETWORKS = panther_config_defaults.INTERNAL_NETWORKS + [ "192.0.2.0/24" ]
```

and at lookup-time:
```
from pypanther.helpers.panther_config import config
print(config.INTERNAL_NETWORKS)
```
prints ["10.0.0.0/8", "192.0.2.0/24" ]
"""



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

teleport_auth_errors_tests: list[RuleTest] = [
    RuleTest(
        name="SSH Errors",
        expected_result=True,
        log={
            "code": "T3007W",
            "error": 'ssh: principal "jack" not in the set of valid principals for given certificate: ["ec2-user"]',
            "event": "auth",
            "success": False,
            "time": "2020-08-13T18:39:42Z",
            "uid": "53e474cc-db1c-45f1-a60d-b31239e20098",
            "user": "panther",
        },
    ),
    RuleTest(
        name="Echo command",
        expected_result=False,
        log={
            "argv": [],
            "cgroup_id": 4294967537,
            "code": "T4000I",
            "ei": 15,
            "event": "session.command",
            "login": "root",
            "namespace": "default",
            "path": "/bin/echo",
            "pid": 7143,
            "ppid": 7115,
            "program": "echo",
            "return_code": 0,
            "server_id": "e75992b4-9e27-456f-b1c9-7a32da83c661",
            "sid": "8a3fc038-785b-43f3-8737-827b3e25fe5b",
            "time": "2020-08-17T17:40:37.491Z",
            "uid": "8eaf8f39-09d4-4a42-a22a-65163d2af702",
            "user": "panther",
        },
    ),
]


@panther_managed
class TeleportAuthErrors(Rule):
    id = "Teleport.AuthErrors-prototype"
    display_name = "Teleport SSH Auth Errors"
    log_types = [LogType.GRAVITATIONAL_TELEPORT_AUDIT]
    tags = ["SSH", "Credential Access:Brute Force"]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0006:T1110"]}
    default_description = "A high volume of SSH errors could indicate a brute-force attack"
    threshold = 10
    dedup_period_minutes = 15
    default_reference = "https://goteleport.com/docs/management/admin/"
    default_runbook = "Check that the user making the failed requests legitimately tried logging in that many times.\n"
    summary_attributes = ["event", "code", "user", "program", "path", "return_code", "login", "server_id", "sid"]
    tests = teleport_auth_errors_tests

    def rule(self, event):
        return bool(event.get("error")) and event.get("event") == "auth"

    def title(self, event):
        return f"A high volume of SSH errors was detected from user [{event.get('user', '<UNKNOWN_USER>')}] on [{event.get('cluster_name', '<UNKNOWN_CLUSTER>')}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

one_login_threshold_accounts_deleted_tests: list[RuleTest] = [
    RuleTest(
        name="Normal User Activated Event",
        expected_result=False,
        log={
            "event_type_id": "16",
            "actor_user_id": 654321,
            "actor_user_name": " Lion",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
    RuleTest(
        name="User Account Delete Event",
        expected_result=True,
        log={
            "event_type_id": "17",
            "actor_user_id": 654321,
            "actor_user_name": " Lion",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
]


@panther_managed
class OneLoginThresholdAccountsDeleted(Rule):
    id = "OneLogin.ThresholdAccountsDeleted-prototype"
    display_name = "OneLogin Multiple Accounts Deleted"
    log_types = [LogType.ONELOGIN_EVENTS]
    tags = ["OneLogin", "Impact:Account Access Removal"]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0040:T1531"]}
    default_description = "Possible Denial of Service detected. Threshold for user account deletions exceeded.\n"
    threshold = 10
    dedup_period_minutes = 10
    default_reference = "https://en.wikipedia.org/wiki/Denial-of-service_attack"
    default_runbook = "Determine if this is normal user-cleanup activity."
    summary_attributes = ["account_id", "user_name", "user_id"]
    tests = one_login_threshold_accounts_deleted_tests

    def rule(self, event):
        # filter events; event type 17 is a user deleted
        return str(event.get("event_type_id")) == "17"

    def title(self, event):
        return (
            f"User [{event.get('actor_user_name', '<UNKNOWN_USER>')}] has exceeded the user account deletion threshold"
        )



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

one_login_user_assumption_tests: list[RuleTest] = [
    RuleTest(
        name="User assumed their own account",
        expected_result=False,
        log={
            "event_type_id": "240",
            "actor_user_id": 123456,
            "actor_user_name": "Bob Cat",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
    RuleTest(
        name="User assumed another user's account",
        expected_result=True,
        log={
            "event_type_id": "3",
            "actor_user_id": 654321,
            "actor_user_name": "Mountain Lion",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
]


@panther_managed
class OneLoginUserAssumption(Rule):
    id = "OneLogin.UserAssumption-prototype"
    display_name = "OneLogin User Assumed Another User"
    log_types = [LogType.ONELOGIN_EVENTS]
    tags = ["OneLogin", "Lateral Movement:Use Alternate Authentication Material"]
    reports = {"MITRE ATT&CK": ["TA0008:T1550"]}
    default_severity = Severity.LOW
    default_description = "User assumed another user account"
    default_reference = "https://onelogin.service-now.com/kb_view_customer.do?sysparm_article=KB0010594#:~:text=Prerequisites,Actions%20and%20select%20Assume%20User."
    default_runbook = "Investigate whether this was authorized access.\n"
    summary_attributes = ["account_id", "user_name", "user_id"]
    tests = one_login_user_assumption_tests

    def rule(self, event):
        # check that this is a user assumption event; event id 3
        return str(event.get("event_type_id")) == "3" and event.get("actor_user_id", "UNKNOWN_USER") != event.get(
            "user_id",
            "UNKNOWN_USER",
        )

    def title(self, event):
        return f"A user [{event.get('actor_user_name', '<UNKNOWN_USER>')}] assumed another user [{event.get('user_name', '<UNKNOWN_USER>')}] account"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_guardduty_context, deep_get

aws_guard_duty_medium_severity_finding_tests: list[RuleTest] = [
    RuleTest(
        name="Medium Sev Finding",
        expected_result=True,
        log={
            "schemaVersion": "2.0",
            "accountId": "123456789012",
            "region": "us-east-1",
            "id": "eeb88ab56556eb7771b266670dddee5a",
            "partition": "aws",
            "arn": "arn:aws:guardduty:us-west-2:123456789012:detector/111111bbbbbbbbbb5555555551111111/finding/90b82273685661b9318f078d0851fe9a",
            "type": "PrivilegeEscalation:IAMUser/AdministrativePermissions",
            "service": {
                "serviceName": "guardduty",
                "detectorId": "111111bbbbbbbbbb5555555551111111",
                "action": {
                    "actionType": "AWS_API_CALL",
                    "awsApiCallAction": {
                        "api": "PutRolePolicy",
                        "serviceName": "iam.amazonaws.com",
                        "callerType": "Domain",
                        "domainDetails": {"domain": "cloudformation.amazonaws.com"},
                        "affectedResources": {"AWS::IAM::Role": "arn:aws:iam::123456789012:role/IAMRole"},
                    },
                },
                "resourceRole": "TARGET",
                "additionalInfo": {},
                "evidence": None,
                "eventFirstSeen": "2020-02-14T17:59:17Z",
                "eventLastSeen": "2020-02-14T17:59:17Z",
                "archived": False,
                "count": 1,
            },
            "severity": 5,
            "createdAt": "2020-02-14T18:12:22.316Z",
            "updatedAt": "2020-02-14T18:12:22.316Z",
            "title": "Principal AssumedRole:IAMRole attempted to add a policy to themselves that is highly permissive.",
            "description": "Principal AssumedRole:IAMRole attempted to add a highly permissive policy to themselves.",
        },
    ),
    RuleTest(
        name="Medium Sev Finding As Sample Data",
        expected_result=False,
        log={
            "schemaVersion": "2.0",
            "accountId": "123456789012",
            "region": "us-east-1",
            "id": "eeb88ab56556eb7771b266670dddee5a",
            "partition": "aws",
            "arn": "arn:aws:guardduty:us-west-2:123456789012:detector/111111bbbbbbbbbb5555555551111111/finding/90b82273685661b9318f078d0851fe9a",
            "type": "PrivilegeEscalation:IAMUser/AdministrativePermissions",
            "service": {
                "serviceName": "guardduty",
                "detectorId": "111111bbbbbbbbbb5555555551111111",
                "action": {
                    "actionType": "AWS_API_CALL",
                    "awsApiCallAction": {
                        "api": "PutRolePolicy",
                        "serviceName": "iam.amazonaws.com",
                        "callerType": "Domain",
                        "domainDetails": {"domain": "cloudformation.amazonaws.com"},
                        "affectedResources": {"AWS::IAM::Role": "arn:aws:iam::123456789012:role/IAMRole"},
                    },
                },
                "resourceRole": "TARGET",
                "additionalInfo": {"sample": True},
                "evidence": None,
                "eventFirstSeen": "2020-02-14T17:59:17Z",
                "eventLastSeen": "2020-02-14T17:59:17Z",
                "archived": False,
                "count": 1,
            },
            "severity": 5,
            "createdAt": "2020-02-14T18:12:22.316Z",
            "updatedAt": "2020-02-14T18:12:22.316Z",
            "title": "Principal AssumedRole:IAMRole attempted to add a policy to themselves that is highly permissive.",
            "description": "Principal AssumedRole:IAMRole attempted to add a highly permissive policy to themselves.",
        },
    ),
]


@panther_managed
class AWSGuardDutyMediumSeverityFinding(Rule):
    id = "AWS.GuardDuty.MediumSeverityFinding-prototype"
    display_name = "AWS GuardDuty Medium Severity Finding"
    log_types = [LogType.AWS_GUARDDUTY]
    tags = ["AWS"]
    default_severity = Severity.MEDIUM
    dedup_period_minutes = 480
    default_description = "A medium-severity GuardDuty finding has been identified.\n"
    default_runbook = "Search related logs to understand the root cause of the activity. Search the Panther Summary Attribute type value in https://docs.aws.amazon.com/guardduty/latest/ug/guardduty_finding-types-active.html for additional details.\n"
    default_reference = (
        "https://docs.aws.amazon.com/guardduty/latest/ug/guardduty_findings.html#guardduty_findings-severity"
    )
    summary_attributes = ["severity", "type", "title", "p_any_domain_names", "p_any_aws_arns", "p_any_aws_account_ids"]
    tests = aws_guard_duty_medium_severity_finding_tests

    def rule(self, event):
        if deep_get(event, "service", "additionalInfo", "sample"):
            # in case of sample data
            # https://docs.aws.amazon.com/guardduty/latest/ug/sample_findings.html
            return False
        return 4.0 <= float(event.get("severity", 0)) <= 6.9

    def title(self, event):
        return event.get("title")

    def alert_context(self, event):
        return aws_guardduty_context(event)
